# -*- coding: utf-8 -*-
# filename: test_utils_async.py
# @Time    : 2025/09/30 23:35
# @Author  : A2C-SMCP
"""
中文：针对 `a2c_smcp/server/utils.py` 的异步工具函数集成测试。
English: Integration tests for async utilities in `a2c_smcp/server/utils.py`.

覆盖点：
- aget_computers_in_office
- aget_all_sessions_in_office
"""

import asyncio

import pytest
from socketio import AsyncClient

from a2c_smcp.server import aget_all_sessions_in_office, aget_computers_in_office
from a2c_smcp.smcp import JOIN_OFFICE_EVENT, SMCP_NAMESPACE


async def _join_office(client: AsyncClient, role: str, office_id: str, name: str) -> None:
    ok, err = await client.call(
        JOIN_OFFICE_EVENT,
        {"role": role, "office_id": office_id, "name": name},
        namespace=SMCP_NAMESPACE,
    )
    assert ok and err is None


@pytest.mark.asyncio
async def test_aget_computers_and_sessions(socketio_server, basic_server_port: int):
    """
    场景：Agent 与 2 个 Computer 加入同一房间，验证工具函数返回。
    """
    # socketio_server fixture 返回的是命名空间，可从中取 server（AsyncServer）
    ns = socketio_server
    sio = ns.server

    agent = AsyncClient()
    comp1 = AsyncClient()
    comp2 = AsyncClient()

    office_id = "office-utils-1"

    # 连接并入场
    await agent.connect(f"http://localhost:{basic_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    await _join_office(agent, role="agent", office_id=office_id, name="robot-U1")

    await comp1.connect(f"http://localhost:{basic_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    await _join_office(comp1, role="computer", office_id=office_id, name="comp-U1")

    await comp2.connect(f"http://localhost:{basic_server_port}", namespaces=[SMCP_NAMESPACE], socketio_path="/socket.io")
    await _join_office(comp2, role="computer", office_id=office_id, name="comp-U2")

    # 等待会话写入完成
    await asyncio.sleep(0.2)

    computers = await aget_computers_in_office(office_id, sio)
    sessions = await aget_all_sessions_in_office(office_id, sio)

    assert len(computers) == 2
    assert all(c["role"] == "computer" for c in computers)
    # 会话应包含3个（1 Agent + 2 Computer）
    assert len(sessions) == 3

    # 清理
    await agent.disconnect()
    await comp1.disconnect()
    await comp2.disconnect()



# -*- coding: utf-8 -*-
# filename: test_agent_desktop.py
# @Time    : 2025/10/05 13:26
# @Author  : A2C-SMCP
# @Email   : qa@a2c-smcp.local
# @Software: PyTest
"""
中文：Agent端桌面协议与事件 单元测试。
English: Unit tests for Agent-side desktop protocol and events.
"""

from typing import Any

import pytest

from a2c_smcp.agent.auth import DefaultAgentAuthProvider
from a2c_smcp.agent.base import BaseAgentClient
from a2c_smcp.agent.client import AsyncSMCPAgentClient
from a2c_smcp.agent.sync_client import SMCPAgentClient
from a2c_smcp.smcp import GET_DESKTOP_EVENT, GetDeskTopReq, GetDeskTopRet


class _DummyAuth(DefaultAgentAuthProvider):
    def __init__(self) -> None:
        super().__init__(agent_id="agent-1", office_id="office-1", api_key=None)


def test_create_get_desktop_request_fields() -> None:
    auth = _DummyAuth()
    base = _make_base_like(auth)
    req = base.create_get_desktop_request("comp-1", size=3, window="window://x")
    assert req["computer"] == "comp-1"
    assert req["agent"] == "agent-1"
    assert isinstance(req["req_id"], str)
    assert req["desktop_size"] == 3
    assert req["window"] == "window://x"


def _make_base_like(auth: _DummyAuth) -> BaseAgentClient:  # type: ignore[return-type]
    class _B(BaseAgentClient):
        def emit(self, event: str, data: Any = None, namespace: str | None = None, callback: Any = None) -> Any:
            return None

        def call(self, event: str, data: Any = None, namespace: str | None = None, timeout: int = 60) -> Any:
            return None

        def register_event_handlers(self) -> None:  # pragma: no cover
            pass

    return _B(auth)


def test_sync_agent_get_desktop_invokes_call(monkeypatch: pytest.MonkeyPatch) -> None:
    auth = _DummyAuth()
    client = SMCPAgentClient(auth_provider=auth)

    called: dict[str, Any] = {}

    def fake_call(event: str, data: GetDeskTopReq, namespace: str | None = None, timeout: int = 60) -> dict:
        called["event"] = event
        called["data"] = data
        return {"desktops": ["window://a"], "req_id": data["req_id"]}

    monkeypatch.setattr(client, "call", fake_call)

    ret = client.get_desktop_from_computer("comp-1", size=2)
    assert called["event"] == GET_DESKTOP_EVENT
    assert called["data"]["computer"] == "comp-1"
    assert called["data"]["agent"] == "agent-1"
    assert ret["desktops"] == ["window://a"]


def test_sync_agent_handle_update_notification(monkeypatch: pytest.MonkeyPatch) -> None:
    auth = _DummyAuth()
    client = SMCPAgentClient(auth_provider=auth)

    fetched: dict[str, Any] = {}

    def fake_get(computer: str, **_: Any) -> GetDeskTopRet:
        fetched["computer"] = computer
        return {"desktops": ["window://b"], "req_id": "x"}

    monkeypatch.setattr(client, "get_desktop_from_computer", fake_get)

    # 直接调用内部回调
    client._on_desktop_updated({"computer": "comp-2"})
    assert fetched["computer"] == "comp-2"


@pytest.mark.asyncio
async def test_async_agent_get_desktop_invokes_call(monkeypatch: pytest.MonkeyPatch) -> None:
    auth = _DummyAuth()
    client = AsyncSMCPAgentClient(auth_provider=auth)

    called: dict[str, Any] = {}

    async def fake_call(event: str, data: GetDeskTopReq, namespace: str | None = None, timeout: int = 60) -> dict:
        called["event"] = event
        called["data"] = data
        return {"desktops": ["window://c"], "req_id": data["req_id"]}

    monkeypatch.setattr(client, "call", fake_call)

    ret = await client.get_desktop_from_computer("comp-3", size=1)
    assert called["event"] == GET_DESKTOP_EVENT
    assert called["data"]["computer"] == "comp-3"
    assert ret["desktops"] == ["window://c"]


@pytest.mark.asyncio
async def test_async_agent_handle_update_notification(monkeypatch: pytest.MonkeyPatch) -> None:
    auth = _DummyAuth()
    client = AsyncSMCPAgentClient(auth_provider=auth)

    fetched: dict[str, Any] = {}

    async def fake_get(computer: str, **_: Any) -> GetDeskTopRet:
        fetched["computer"] = computer
        return {"desktops": ["window://d"], "req_id": "y"}

    monkeypatch.setattr(client, "get_desktop_from_computer", fake_get)

    await client._on_desktop_updated({"computer": "comp-4"})
    assert fetched["computer"] == "comp-4"



"""
* 文件名: test_smcp_namespace_sync
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: pytest, socketio
* 描述: 同步版 SMCP Namespace 测试用例 / Sync SMCP Namespace test cases
"""

from unittest.mock import MagicMock

import pytest

from a2c_smcp.server import (
    DefaultSyncAuthenticationProvider,
    SyncAuthenticationProvider,
    SyncSMCPNamespace,
)
from a2c_smcp.smcp import SMCP_NAMESPACE, EnterOfficeReq, LeaveOfficeReq


class MockSyncAuthProvider(SyncAuthenticationProvider):
    """Mock同步认证提供者 / Mock sync authentication provider"""

    def authenticate(self, sio, environ: dict, auth: dict | None, headers: list) -> bool:  # noqa: D401
        for header in headers:
            if isinstance(header, (list, tuple)) and len(header) >= 2:
                header_name = header[0].decode("utf-8").lower() if isinstance(header[0], bytes) else str(header[0]).lower()
                header_value = header[1].decode("utf-8") if isinstance(header[1], bytes) else str(header[1])
                if header_name == "x-api-key" and header_value == "valid_key":
                    return True
        return False


@pytest.fixture
def mock_auth_provider():
    return MockSyncAuthProvider()


@pytest.fixture
def smcp_namespace(mock_auth_provider):
    return SyncSMCPNamespace(mock_auth_provider)


@pytest.fixture
def mock_server():
    server = MagicMock()
    server.app = MagicMock()
    server.app.state = MagicMock()
    server.app.state.agent_id = "test_agent"
    server.manager = MagicMock()
    # get_participants 返回空列表
    server.manager.get_participants.return_value = []
    return server


class TestSyncSMCPNamespace:
    def test_namespace_initialization(self, smcp_namespace):
        assert smcp_namespace.namespace == SMCP_NAMESPACE
        assert isinstance(smcp_namespace.auth_provider, MockSyncAuthProvider)

    def test_successful_connection(self, smcp_namespace, mock_server):
        smcp_namespace.server = mock_server

        environ = {
            "asgi": {
                "scope": {
                    "headers": [
                        (b"x-api-key", b"valid_key"),
                    ],
                },
            },
        }

        result = smcp_namespace.on_connect("test_sid", environ, None)
        assert result is True

    def test_failed_authentication(self, smcp_namespace, mock_server):
        smcp_namespace.server = mock_server

        environ = {
            "asgi": {
                "scope": {
                    "headers": [
                        (b"x-api-key", b"invalid_key"),
                    ],
                },
            },
        }

        with pytest.raises(ConnectionRefusedError):
            smcp_namespace.on_connect("test_sid", environ, None)

    def test_join_office_success(self, smcp_namespace):
        # mock 会话相关方法
        session = {}
        smcp_namespace.get_session = MagicMock(return_value=session)
        smcp_namespace.save_session = MagicMock()
        smcp_namespace.enter_room = MagicMock()

        data = EnterOfficeReq(**{
            "role": "computer",
            "name": "test_computer",
            "office_id": "office_123",
        })

        success, error = smcp_namespace.on_server_join_office("test_sid", data)

        assert success is True
        assert error is None
        assert session["role"] == "computer"
        assert session["name"] == "test_computer"

    def test_join_office_role_mismatch(self, smcp_namespace):
        session = {"role": "agent"}
        smcp_namespace.get_session = MagicMock(return_value=session)
        smcp_namespace.save_session = MagicMock()

        data = EnterOfficeReq(**{
            "role": "computer",
            "name": "test_computer",
            "office_id": "office_123",
        })

        success, error = smcp_namespace.on_server_join_office("test_sid", data)

        assert success is False
        assert "Role mismatch" in error

    def test_leave_office(self, smcp_namespace):
        smcp_namespace.leave_room = MagicMock()

        data = LeaveOfficeReq(**{"office_id": "office_123"})

        success, error = smcp_namespace.on_server_leave_office("test_sid", data)

        assert success is True
        assert error is None
        smcp_namespace.leave_room.assert_called_once_with("test_sid", "office_123")



# -*- coding: utf-8 -*-
# filename: conftest.py
# @Time    : 2025/9/30 16:55
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文：集成测试全局fixtures，提供Socket.IO测试服务器与端口。
English: Global fixtures for integration tests, providing Socket.IO test server and free port.
"""

import socket
from collections.abc import AsyncGenerator

import pytest
from socketio import ASGIApp

from a2c_smcp.smcp import SMCP_NAMESPACE
from tests.integration_tests.computer.socketio.mock_uv_server import UvicornTestServer
from tests.integration_tests.mock_socketio_server import MockComputerServerNamespace, create_computer_test_socketio


@pytest.fixture
def basic_server_port() -> int:
    """
    中文：查找可用端口。
    English: Find an available TCP port.
    """
    with socket.socket() as s:
        s.bind(("127.0.0.1", 0))
        return s.getsockname()[1]


@pytest.fixture
async def socketio_server(basic_server_port: int) -> AsyncGenerator[MockComputerServerNamespace, None]:
    """
    中文：启动基于标准SMCPNamespace的测试服务器，返回命名空间以便测试访问。
    English: Start test server based on standard SMCPNamespace and return the namespace for test access.
    """
    sio = create_computer_test_socketio()
    # 避免关闭时后台任务异常 / avoid background task issues on shutdown
    sio.eio.start_service_task = False
    asgi_app = ASGIApp(sio, socketio_path="/socket.io")

    server = UvicornTestServer(asgi_app, port=basic_server_port)
    await server.up()
    try:
        yield sio.namespace_handlers[SMCP_NAMESPACE]  # type: ignore[index]
    finally:
        # 强制快速关闭，不等待连接清理 / Force fast shutdown without waiting for connection cleanup
        await server.down(force=True)



# -*- coding: utf-8 -*-
# filename: base_client.pyi
# @Time    : 2025/8/19 10:28
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
import asyncio
from abc import ABC, abstractmethod
from collections.abc import Awaitable
from collections.abc import Callable as Callable
from contextlib import AsyncExitStack
from enum import StrEnum
from typing import Any, Generic, TypeVar

from mcp import ClientSession
from mcp import Tool as Tool
from mcp.client.session import MessageHandlerFnT
from mcp.types import CallToolResult, InitializeResult, ReadResourceResult, Resource
from pydantic import BaseModel as BaseModel
from transitions.core import EventData
from transitions.extensions.asyncio import AsyncMachine, AsyncTransitionConfigDict

from a2c_smcp.utils.async_property import async_property

# 泛型参数，用于约束 MCP Server 参数类型
# Generic parameter for constraining MCP Server parameter types
ParamsT = TypeVar("ParamsT", bound=BaseModel)

class STATES(StrEnum):
    initialized = "initialized"
    connected = "connected"
    disconnected = "disconnected"
    error = "error"

TRANSITIONS: list[AsyncTransitionConfigDict]

class A2CAsyncMachine(AsyncMachine): ...

class BaseMCPClient(ABC, Generic[ParamsT]):
    params: ParamsT
    _aexit_stack: AsyncExitStack
    machine: A2CAsyncMachine
    _async_session: ClientSession | None
    _state_change_callback: Callable[[str, str], None | Awaitable[None]] | None
    _message_handler: MessageHandlerFnT | None
    _session_keep_alive_task: asyncio.Task | None
    _create_session_success_event: asyncio.Event
    _create_session_failure_event: asyncio.Event
    _async_session_closed_event: asyncio.Event
    _initialize_result: InitializeResult | None
    state: STATES

    def __init__(
        self,
        params: ParamsT,
        state_change_callback: Callable[[str, str], None | Awaitable[None]] | None = None,
        message_handler: MessageHandlerFnT | None = None,
    ) -> None: ...
    async def _trigger_state_change(self, event: EventData) -> None: ...
    @async_property
    async def async_session(self) -> ClientSession: ...
    @abstractmethod
    async def _create_async_session(self) -> ClientSession: ...
    @property
    def initialize_result(self) -> InitializeResult | None: ...
    async def aconnect(self, *args: Any, **kwargs: Any) -> None: ...
    async def aprepare_connect(self, event: EventData) -> None: ...
    async def acan_connect(self, event: EventData) -> bool: ...
    async def abefore_connect(self, event: EventData) -> None: ...
    async def on_enter_connected(self, event: EventData) -> None: ...
    async def aafter_connect(self, event: EventData) -> None: ...
    async def adisconnect(self, *args: Any, **kwargs: Any) -> None: ...
    async def aprepare_disconnect(self, event: EventData) -> None: ...
    async def acan_disconnect(self, event: EventData) -> bool: ...
    async def abefore_disconnect(self, event: EventData) -> None: ...
    async def on_enter_disconnected(self, event: EventData) -> None: ...
    async def aafter_disconnect(self, event: EventData) -> None: ...
    async def aerror(self, *args: Any, **kwargs: Any) -> None: ...
    async def aprepare_error(self, event: EventData) -> None: ...
    async def acan_error(self, event: EventData) -> bool: ...
    async def abefore_error(self, event: EventData) -> None: ...
    async def on_enter_error(self, event: EventData) -> None: ...
    async def aafter_error(self, event: EventData) -> None: ...
    async def ainitialize(self, *args: Any, **kwargs: Any) -> None: ...
    async def aprepare_initialize(self, event: EventData) -> None: ...
    async def acan_initialize(self, event: EventData) -> bool: ...
    async def abefore_initialize(self, event: EventData) -> None: ...
    async def on_enter_initialized(self, event: EventData) -> None: ...
    async def aafter_initialize(self, event: EventData) -> None: ...
    async def list_tools(self) -> list[Tool]: ...
    async def list_windows(self) -> list[Resource]: ...
    async def get_window_detail(self, resource: Resource | str) -> ReadResourceResult: ...
    async def call_tool(self, tool_name: str, params: dict) -> CallToolResult: ...
    async def _keep_alive_task(self) -> None: ...
    async def _close_task(self) -> None: ...



# -*- coding: utf-8 -*-
# filename: smcp.py
# @Time    : 2025/8/8 12:35
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from typing import Any, Literal, NotRequired, TypeAlias

from typing_extensions import TypedDict

from a2c_smcp import types
from a2c_smcp.types import SERVER_NAME, TOOL_NAME

SMCP_NAMESPACE = "/smcp"
# 除了notify:外的所有事件 服务端必须实现，因为由服务端转换或者执行完毕。而notify:*事件均由Server发出，因此Server不需要实现
# 客户端事件 由client:开头的事件ComputerClient必须全部实现，一般由AgentClient触发，由Server转发。client: 开头的事件，会由特定的某一个
# ComputerClient执行
# 一般data中会明确指定computer的sid用于执行此事件，如果需要多个client执行，一般是通过server:事件触发，广播至房间内的所有Computer
TOOL_CALL_EVENT = "client:tool_call"
GET_CONFIG_EVENT = "client:get_config"
GET_TOOLS_EVENT = "client:get_tools"
GET_DESKTOP_EVENT = "client:get_desktop"
# 服务端事件 由server:开头的事件服务端执行
JOIN_OFFICE_EVENT = "server:join_office"
LEAVE_OFFICE_EVENT = "server:leave_office"
UPDATE_CONFIG_EVENT = "server:update_config"
UPDATE_TOOL_LIST_EVENT = "server:update_tool_list"
# 桌面刷新事件：当资源列表或资源内容变化时，由Computer端通知Server广播。
# 中文: 当需要让Agent刷新桌面时，由Computer触发此事件。英文: Computer emits this when Agent should refresh desktop.
UPDATE_DESKTOP_EVENT = "server:update_desktop"
CANCEL_TOOL_CALL_EVENT = "server:tool_call_cancel"
# 列出房间内所有会话事件：Agent可以通过此事件查询指定房间内的所有会话信息。
# List all sessions in room event: Agent can query all session info in a specific room via this event.
LIST_ROOM_EVENT = "server:list_room"
# NOTIFY 通知事件  通知事件全部由Server发出（一般由Client触发其它事件，在响应这些事件时，Server发出通知）
#   1. 比如 AgentClient 发出 server:tool_call_cancel 事件，服务端接收后，发起 notify:tool_call_cancel 通知
#   2. 比如 ComputerClient 发出 server:join_office 事件，服务端接收后，发起 notify:enter_office 通知
# AgentClient与ComputerClient选择性接收。因为Notify均由Server发出，因此Server中不需要实现对应接收方法
CANCEL_TOOL_CALL_NOTIFICATION = "notify:tool_call_cancel"
ENTER_OFFICE_NOTIFICATION = "notify:enter_office"  # AgentClient必须实现 以此，配合 client:get_config 与 client:get_tools 更新工具配置
LEAVE_OFFICE_NOTIFICATION = "notify:leave_office"  # AgentClient必须实现 以此，配合 client:get_config 与 client:get_tools 更新工具配置
UPDATE_CONFIG_NOTIFICATION = "notify:update_config"  # AgentClient必须实现 以此，配合 client:get_config
UPDATE_TOOL_LIST_NOTIFICATION = "notify:update_tool_list"  # AgentClient必须实现，通过 client:get_tools 实现更新工具配置
# 桌面刷新通知：Server接收 server:update_desktop 后广播。Agent据此拉取最新桌面。
UPDATE_DESKTOP_NOTIFICATION = "notify:update_desktop"


class AgentCallData(TypedDict):
    agent: str  # agent name
    req_id: str


class ToolCallReq(AgentCallData):
    computer: str
    tool_name: str
    params: dict
    timeout: int


class GetToolsReq(AgentCallData):
    computer: str  # 计算机名称


class SMCPTool(TypedDict):
    """在Computer端侧管理多个MCP时，无法保证ToolName不重复。因此alias字段被添加以帮助用户进行区分不同工具。如果alias被设置，创建工具时将会使用alias。"""

    name: str
    description: str
    params_schema: dict
    return_schema: dict | None
    meta: NotRequired[types.Attributes | None]


class GetToolsRet(TypedDict):
    tools: list[SMCPTool]
    req_id: str


class EnterOfficeReq(TypedDict):
    role: Literal["computer", "agent"]
    name: str
    office_id: str


class LeaveOfficeReq(TypedDict):
    office_id: str


class UpdateComputerConfigReq(TypedDict):
    computer: str  # 机器人计算机自定义名称


class GetComputerConfigReq(AgentCallData):
    computer: str  # 机器人计算机自定义名称


class ToolMeta(TypedDict, total=False):
    auto_apply: NotRequired[bool | None]
    # 不同MCP工具返回值并不统一，虽然其满足MCP标准的返回格式，但具体的原始内容命名仍然无法避免出现不一致的情况。通过object_mapper可以方便
    # 前端对其进行转换，以使用标准组件渲染解析。
    ret_object_mapper: NotRequired[dict | None]
    # 工具别名，与 model.ToolMeta.alias 对齐，用于解决不同 Server 下工具重名冲突
    # Tool alias, align with model.ToolMeta.alias, used to resolve name conflicts across servers
    alias: NotRequired[str | None]
    # 工具标签，用于对工具进行分类
    # Tool tags, used to categorize tools
    tags: NotRequired[list[str] | None]


class BaseMCPServerConfig(TypedDict):
    """MCP服务器配置基类"""

    name: SERVER_NAME  # MCP Server的名称
    disabled: bool
    forbidden_tools: list[str]  # 禁用的工具列表，因为一个mcp可能有非常多工具，有些工具用户需要禁用。
    tool_meta: dict[TOOL_NAME, ToolMeta]
    # 默认工具元数据（可选）。当某个具体工具未在 tool_meta 中提供专门配置时，使用该默认配置。
    # Default tool metadata (optional). Used when a specific tool has no explicit entry in tool_meta.
    default_tool_meta: NotRequired[ToolMeta | None]
    # VRL脚本（可选）。用于对工具返回值进行动态转换和格式化。如果配置了VRL脚本，在初始化时会进行语法检查。
    # VRL script (optional). Used to dynamically transform and format tool return values. Syntax check on initialization.
    vrl: NotRequired[str | None]


# --- MCPServer 配置，参考借鉴： ---
# VSCode: https://code.visualstudio.com/docs/copilot/chat/mcp-servers#_configuration-format
# AWS Q Developer: https://docs.aws.amazon.com/amazonq/latest/qdeveloper-ug/mcp-ide.html


class MCPServerInputBase(TypedDict):
    """MCP服务器输入配置基类"""

    id: str
    description: str


class MCPServerPromptStringInput(MCPServerInputBase):
    """字符串输入类型，参考：https://code.visualstudio.com/docs/reference/variables-reference#_input-variables"""

    type: Literal["promptString"]
    default: NotRequired[str | None]
    password: NotRequired[bool | None]


class MCPServerPickStringInput(MCPServerInputBase):
    """选择输入类型，参考：https://code.visualstudio.com/docs/reference/variables-reference#_input-variables"""

    type: Literal["pickString"]
    options: list[str]
    default: NotRequired[str | None]


class MCPServerCommandInput(MCPServerInputBase):
    """命令输入类型，参考：https://code.visualstudio.com/docs/reference/variables-reference#_input-variables"""

    type: Literal["command"]
    command: str
    args: NotRequired[dict[str, str] | None]


MCPServerInput = MCPServerPromptStringInput | MCPServerPickStringInput | MCPServerCommandInput


class MCPServerStdioParameters(TypedDict):
    command: str
    """The executable to run to start the server."""
    args: list[str]
    """Command line arguments to pass to the executable."""
    env: dict[str, str] | None
    """
    The environment to use when spawning the process.

    If not specified, the result of get_default_environment() will be used.
    """
    cwd: str | None
    """The working directory to use when spawning the process."""
    encoding: str
    """
    The text encoding used when sending/receiving messages to the server

    defaults to utf-8
    """
    encoding_error_handler: Literal["strict", "ignore", "replace"]
    """
    The text encoding error handler.

    See https://docs.python.org/3/library/codecs.html#codec-base-classes for
    explanations of possible values
    """


class MCPServerStdioConfig(BaseMCPServerConfig):
    """标准输入输出模式的MCP服务器配置"""

    type: Literal["stdio"]
    server_parameters: MCPServerStdioParameters


# !!! 注意在 MCP 官方 python-client 中有一个奇怪的问题 StreamableServerParams中 timeout 与 sse_read_timeout 是使用 timedelta 定义的，这个
# 定义会在序列化时自动转义为字符串（基于 ISO 8601）但是SSEServerParams，它直接使用float定义，因此这里需要适配一下。导致了两个定义名称一致，
# 但数据结构不一致。


class MCPServerStreamableHttpParameters(TypedDict):
    # The endpoint URL.
    url: str
    # Optional headers to include in requests.
    headers: dict[str, Any] | None
    # HTTP timeout for regular operations.
    timeout: str
    # Timeout for SSE read operations.
    sse_read_timeout: str
    # Close the client session when the transport closes.
    terminate_on_close: bool


class MCPServerStreamableHttpConfig(BaseMCPServerConfig):
    """StreamableHttpHTTP模式的MCP服务器配置"""

    type: Literal["streamable"]
    server_parameters: MCPServerStreamableHttpParameters


class MCPSSEParameters(TypedDict):
    # The endpoint URL.
    url: str
    # Optional headers to include in requests.
    headers: dict[str, Any] | None
    # HTTP timeout for regular operations.
    timeout: float
    # Timeout for SSE read operations.
    sse_read_timeout: float


class MCPSSEConfig(BaseMCPServerConfig):
    """SSE模式的MCP服务器配置"""

    type: Literal["sse"]
    server_parameters: MCPSSEParameters


MCPServerConfig = MCPServerStdioConfig | MCPServerStreamableHttpConfig | MCPSSEConfig


class GetComputerConfigRet(TypedDict):
    """完整的Computer配置文件类型"""

    inputs: NotRequired[list[MCPServerInput] | None]
    servers: dict[str, MCPServerConfig]


class LeaveOfficeNotification(TypedDict, total=False):
    """Agent或者Computer离开房间的通知，需要向房间内其他人广播。广播时间为真实离开之前，也就是即将离开"""

    office_id: str
    computer: str | None
    agent: str | None


class EnterOfficeNotification(TypedDict, total=False):
    """Agent或者Computer加入房间的通知，需要向房间内其他人广播。广播时间为真实加入之后"""

    office_id: str
    computer: str | None  # Computer Name
    agent: str | None  # Agent Name


class UpdateMCPConfigNotification(TypedDict, total=False):
    """
    MCP配置更新的通知，需要向房间内其他人广播
    """

    computer: str  # 机器人计算机自定义名称


class UpdateToolListNotification(TypedDict, total=False):
    """
    工具列表更新的通知，需要向房间内其他人广播。
    Notification of tool list update, should be broadcast to others in the room.
    """

    computer: str  # 机器人计算机自定义名称


class GetDeskTopReq(AgentCallData, total=True):
    """
    获取当前Computer的桌面信息。
    """

    computer: str  # computer name
    desktop_size: NotRequired[int]  # 指定希望获取的桌面内容长度。如果不指定，则会全量返回，由调用方进行处理。
    window: NotRequired[str]  # 指定获取的WindowURI，如果不指定则由Desktop自动组织，如果指定，会尝试获取指定的Window


Desktop: TypeAlias = str


class GetDeskTopRet(TypedDict, total=False):
    """
    Computer的桌面布局与内容信息。Agent可以通过相应指令来获取。
    The layout and content on Computer. Agent get it by spec event.
    """

    desktops: list[Desktop]
    req_id: str


class ListRoomReq(AgentCallData):
    """
    列出房间内所有会话的请求。Agent通过此请求获取指定房间内的所有Computer和Agent会话信息。
    Request to list all sessions in a room. Agent uses this to get all Computer and Agent session info in a specific room.
    """

    office_id: str  # 房间ID / Room ID


class SessionInfo(TypedDict, total=False):
    """
    会话信息，包含sid、name、role等基础信息。
    Session information, includes sid, name, role and other basic info.
    """

    sid: str  # 会话ID / Session ID
    name: str  # 会话名称 / Session name
    role: Literal["computer", "agent"]  # 会话角色 / Session role
    office_id: str  # 所属房间ID / Office ID


class ListRoomRet(TypedDict):
    """
    列出房间内所有会话的响应。返回房间内所有Computer和Agent的会话信息列表。
    Response for listing all sessions in a room. Returns list of all Computer and Agent session info in the room.
    """

    sessions: list[SessionInfo]  # 会话列表 / Session list
    req_id: str  # 请求ID / Request ID



# -*- coding: utf-8 -*-
# filename: mock_sync_smcp_server.py
# @Time    : 2025/9/30 22:50
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文：同步 SMCP 服务器 Mock 实现，用于同步客户端集成测试。
English: Synchronous SMCP server Mock implementation for sync client integration tests.
"""

from typing import Any

from mcp.types import CallToolResult, TextContent
from socketio import Namespace, Server

from a2c_smcp.smcp import (
    ENTER_OFFICE_NOTIFICATION,
    SMCP_NAMESPACE,
    UPDATE_CONFIG_NOTIFICATION,
    UPDATE_DESKTOP_NOTIFICATION,
    EnterOfficeNotification,
    EnterOfficeReq,
    GetDeskTopReq,
    GetDeskTopRet,
    GetToolsReq,
    GetToolsRet,
    ListRoomReq,
    ListRoomRet,
    SessionInfo,
    SMCPTool,
    ToolCallReq,
    UpdateMCPConfigNotification,
)
from a2c_smcp.utils.logger import logger


class MockSyncSMCPNamespace(Namespace):
    """
    中文：同步 SMCP 命名空间 Mock 实现。
    English: Synchronous SMCP namespace Mock implementation.
    """

    def __init__(self) -> None:
        super().__init__(namespace=SMCP_NAMESPACE)
        # 存储会话信息：sid -> {role, name, office_id}
        # Store session info: sid -> {role, name, office_id}
        self.sessions: dict[str, dict[str, Any]] = {}

    def trigger_event(self, event: str, *args: Any) -> Any:
        """触发事件，重写触发逻辑，将冒号转换为下划线"""
        return super().trigger_event(event.replace(":", "_"), *args)

    def on_connect(self, sid: str, environ: dict, auth: dict | None = None) -> bool:
        logger.info(f"SocketIO Client {sid} connecting...")
        return True

    def on_disconnect(self, sid: str) -> None:
        logger.info(f"SocketIO Client {sid} disconnected")
        # 清理会话信息 / Clean up session info
        if sid in self.sessions:
            del self.sessions[sid]

    def on_server_join_office(self, sid: str, data: EnterOfficeReq) -> tuple[bool, str | None]:
        """处理加入办公室请求"""
        logger.info(f"Computer/Agent {sid} 加入房间 {data['office_id']}")

        # 存储会话信息 / Store session info
        self.sessions[sid] = {
            "sid": sid,
            "role": data["role"],
            "name": data["name"],
            "office_id": data["office_id"],
        }

        self.enter_room(sid, data["office_id"])

        # 广播进入办公室通知
        notification = EnterOfficeNotification(
            office_id=data["office_id"],
            computer=sid if data["role"] == "computer" else None,
            agent=sid if data["role"] == "agent" else None,
        )

        self.emit(
            ENTER_OFFICE_NOTIFICATION,
            notification,
            skip_sid=sid,
            room=data["office_id"],
        )
        return True, "加入成功"

    def on_server_update_config(self, sid: str, data: dict) -> tuple[bool, str | None]:
        """处理更新配置请求"""
        logger.info(f"Computer {sid} 更新配置")
        computer = data.get("computer", sid)

        # 广播配置更新通知
        notification = UpdateMCPConfigNotification(computer=computer)
        self.emit(UPDATE_CONFIG_NOTIFICATION, notification, skip_sid=sid)
        return True, "配置更新成功"

    def on_client_tool_call(self, sid: str, data: ToolCallReq) -> dict:
        """处理工具调用请求"""
        logger.info(f"Agent {sid} 调用工具 {data['tool_name']}")

        # 返回模拟的工具调用结果
        result = CallToolResult(
            isError=False,
            content=[TextContent(type="text", text="mock tool result")],
        )
        return result.model_dump(mode="json")

    def on_client_get_tools(self, sid: str, data: GetToolsReq) -> GetToolsRet:
        """处理获取工具列表请求"""
        logger.info(f"Agent {sid} 拉取工具列表")

        # 返回模拟的工具列表
        tools = [
            SMCPTool(
                name="echo",
                description="echo text",
                params_schema={"type": "object", "properties": {"text": {"type": "string"}}},
                return_schema=None,
            ),
            SMCPTool(
                name="test_tool",
                description="test tool",
                params_schema={},
                return_schema=None,
            ),
        ]

        return GetToolsRet(tools=tools, req_id=data["req_id"])

    def on_client_get_desktop(self, sid: str, data: GetDeskTopReq) -> GetDeskTopRet:
        """处理获取桌面请求（返回固定桌面数据）。"""
        logger.info(f"Agent {sid} 拉取桌面数据 size={data.get('desktop_size')}")
        desktops = ["window://mock\n\nhello world"]
        return GetDeskTopRet(desktops=desktops, req_id=data["req_id"])

    def on_server_update_desktop(self, sid: str, data: dict) -> tuple[bool, str | None]:
        """处理桌面更新请求并广播通知。"""
        logger.info(f"Computer {sid} 请求广播桌面更新")
        computer = data.get("computer", sid)
        self.emit(UPDATE_DESKTOP_NOTIFICATION, {"computer": computer}, skip_sid=sid)
        return True, None

    def on_server_list_room(self, sid: str, data: ListRoomReq) -> ListRoomRet:
        """
        列出指定房间内的所有会话信息
        List all sessions in the specified room

        Args:
            sid (str): 发起者ID，一般是Agent / Initiator ID, usually Agent
            data (ListRoomReq): 列出房间请求数据 / List room request data

        Returns:
            ListRoomRet: 房间内所有会话信息列表 / List of all session info in the room
        """
        office_id = data["office_id"]
        req_id = data["req_id"]

        logger.info(f"Agent {sid} 查询房间 {office_id} 的会话列表")

        # 过滤出指定房间内的所有会话 / Filter all sessions in the specified room
        sessions: list[SessionInfo] = []
        for _session_sid, session_data in self.sessions.items():
            if session_data.get("office_id") == office_id:
                session_info: SessionInfo = {
                    "sid": session_data["sid"],
                    "name": session_data["name"],
                    "role": session_data["role"],
                    "office_id": session_data["office_id"],
                }
                sessions.append(session_info)

        logger.info(f"房间 {office_id} 中找到 {len(sessions)} 个会话")
        return ListRoomRet(sessions=sessions, req_id=req_id)


def create_sync_smcp_socketio() -> Server:
    """
    创建同步 SMCP Socket.IO 服务器
    Create synchronous SMCP Socket.IO server

    Returns:
        Server: Socket.IO 服务器实例
    """
    sio = Server(
        cors_allowed_origins="*",
        ping_timeout=60,
        ping_interval=25,
        async_handlers=False,  # 重要：使用同步处理器
        always_connect=True,
    )

    # 注册 SMCP 命名空间
    sio.register_namespace(MockSyncSMCPNamespace())

    return sio



# filename: manager.py
# @Time    : 2025/8/17 16:53
# @Author  : JQQ
# @Email   : jiaqia@qknode.com
# @Software: PyCharm
import asyncio
import copy
import json
from collections import defaultdict
from collections.abc import AsyncGenerator, Iterable
from typing import Any

from mcp.client.session import MessageHandlerFnT
from mcp.types import CallToolResult, ReadResourceResult, Resource, Tool
from vrl_python import VRLRuntime

from a2c_smcp.computer.mcp_clients.model import A2C_TOOL_META, A2C_VRL_TRANSFORMED, MCPClientProtocol, MCPServerConfig, ToolMeta
from a2c_smcp.computer.mcp_clients.utils import client_factory
from a2c_smcp.types import SERVER_NAME, TOOL_NAME
from a2c_smcp.utils.logger import logger


class ToolNameDuplicatedError(Exception):
    def __init__(self, *args: Any) -> None:
        super().__init__(*args)


class MCPServerManager:
    """
    MCP Server管理器

    所有以下划线开头的私有方法是非协程安全的。如果外部调用，需要使用普通方法。

    # TODO call_tool的时候需要启动子任务对call_tool进行包装，以便进行超时控制与动态取消。实现超时控制与动态取消后，才可以响应服务端取消任务
    """

    def __init__(
        self,
        auto_connect: bool = False,
        auto_reconnect: bool = True,
        message_handler: MessageHandlerFnT | None = None,
    ) -> None:
        # 存储所有服务器配置
        self._servers_config: dict[SERVER_NAME, MCPServerConfig] = {}
        # 存储活动客户端 {server_name: client}
        self._active_clients: dict[SERVER_NAME, MCPClientProtocol] = {}
        # 工具到服务器的映射 {tool_name: server_name}
        self._tool_mapping: dict[TOOL_NAME, SERVER_NAME] = {}
        # 工具的alias到server+original_name的映射 {alias: (server_name, original_name)}
        self._alias_mapping: dict[str, tuple[SERVER_NAME, TOOL_NAME]] = {}
        # 禁用工具集合
        self._disabled_tools: set[TOOL_NAME] = set()
        # 自动重连标志
        self._auto_reconnect: bool = auto_reconnect
        # 自动连接标志
        self._auto_connect: bool = auto_connect
        # 自定义消息处理器，透传到各具体Client
        self._message_handler: MessageHandlerFnT | None = message_handler
        # 内部锁防止并发修改
        self._lock = asyncio.Lock()

    def get_server_config(self, server_name: SERVER_NAME) -> MCPServerConfig:
        """通过名称获取服务配置"""
        return self._servers_config[server_name]

    def get_tool_meta(self, server_name: SERVER_NAME, tool_name: TOOL_NAME) -> ToolMeta | None:
        """
        中文: 获取指定服务器下某工具合并后的元数据（优先具体 tool_meta，缺失字段回落 default_tool_meta）。
        English: Get merged ToolMeta for a tool under the given server (specific overrides; fallback to default).

        Args:
            server_name (SERVER_NAME): 服务器名称 / server name
            tool_name (TOOL_NAME): 工具原始名称或别名解析后的名称 / tool name

        Returns:
            ToolMeta | None: 合并后的工具元数据；若两侧均为空返回 None / merged ToolMeta or None if both absent.
        """
        config = self.get_server_config(server_name)
        return self._merged_tool_meta(config, tool_name)

    async def enable_auto_connect(self) -> None:
        """启用自动连接"""
        async with self._lock:
            self._auto_connect = True

    async def disable_auto_connect(self) -> None:
        """禁用自动连接"""
        async with self._lock:
            self._auto_connect = False

    async def enable_auto_reconnect(self) -> None:
        """启用自动重连"""
        async with self._lock:
            self._auto_reconnect = True

    async def disable_auto_reconnect(self) -> None:
        """禁用自动重连"""
        async with self._lock:
            self._auto_reconnect = False

    async def ainitialize(self, servers: Iterable[MCPServerConfig]) -> None:
        """
        初始化管理器并添加服务器配置

        Args:
            servers (list[MCPServerConfig]): MCP服务器配置
        """
        async with self._lock:
            # 清理旧设置与配置
            # 1. 停止所有活动客户端
            await self._astop_all()
            # 2. 清空所有状态存储
            self._clear_all()
            # 3. 添加新配置
            for server in servers:
                await self._add_or_update_server_config(server)
            try:
                await self._arefresh_tool_mapping()
            except ToolNameDuplicatedError as e:  # pragma: no cover
                # 极端分支：仅在外部错误用法下触发，主流程不会走到这里
                # 中文：此处为防御性分支，正常流程不会触发
                # English: Defensive branch, not triggered in normal flow
                await self._astop_all()  # pragma: no cover
                self._clear_all()  # pragma: no cover
                raise e  # pragma: no cover

    async def _add_or_update_server_config(self, config: MCPServerConfig) -> None:
        """
        添加/更新服务器配置（不启动客户端）

        如果已存在，检查是否已经建立客户端连接，如果是，检查是否需要自动重连
        如果不存在，直接添加配置

        Args:
            config (MCPServerConfig): MCP服务器配置
        """
        if config.name in self._servers_config:
            # 配置更新时检查是否激活
            if config.name in self._active_clients:
                if self._auto_reconnect:
                    self._servers_config[config.name] = config
                    await self._arestart_server(config.name)
                else:
                    raise RuntimeError(f"Server {config.name} is active. Stop it before updating config")
            else:
                # 配置存在但客户端未激活，更新配置并根据 auto_connect 决定是否启动
                # Config exists but client is not active, update config and start if auto_connect is enabled
                self._servers_config[config.name] = config
                if self._auto_connect:
                    await self._astart_client(config.name)
        else:
            self._servers_config[config.name] = config
            if self._auto_connect:
                await self._astart_client(config.name)

    async def aadd_or_aupdate_server(self, config: MCPServerConfig) -> None:
        """
        添加或更新服务器配置

        Args:
            config (MCPServerConfig): MCP服务器配置
        """
        async with self._lock:
            backup_config = copy.deepcopy(self._servers_config)
            try:
                await self._add_or_update_server_config(config)
                await self._arefresh_tool_mapping()
            except ToolNameDuplicatedError as e:
                self._servers_config = backup_config
                raise e

    async def aremove_server(self, server_name: str) -> None:
        """移除服务器配置"""
        async with self._lock:
            if server_name in self._active_clients:
                await self._astop_client(server_name)
            del self._servers_config[server_name]
            await self._arefresh_tool_mapping()

    async def _arestart_server(self, server_name: str) -> None:
        """
        重启服务器客户端

        Args:
            server_name (str): 服务器名称
        """
        # 明确使用当前管理器中的最新配置
        config = self._servers_config.get(server_name)
        if not config:
            # 极端分支：仅在外部错误用法下触发，主流程不会走到这里
            # 中文：此处为防御性分支，正常流程不会触发
            # English: Defensive branch, not triggered in normal flow
            raise ValueError(f"Server {server_name} not found in config")  # pragma: no cover

        # 确保使用最新配置重启
        if server_name in self._active_clients:
            await self._astop_client(server_name)

        # 只有启用的配置才能重启
        if not config.disabled:
            await self._astart_client(server_name)

    async def astart_all(self) -> None:
        """启动所有启用的服务器"""
        async with self._lock:
            logger.debug(f"Manager Start all async task: {asyncio.current_task()}")
            for server_name in self._servers_config:
                if not self._servers_config[server_name].disabled:
                    await self._astart_client(server_name)

    async def astart_client(self, server_name: str) -> None:
        """启动单个服务器客户端"""
        async with self._lock:
            await self._astart_client(server_name)

    async def _astart_client(self, server_name: str) -> None:
        """启动单个服务器客户端"""
        config = self._servers_config.get(server_name)
        if not config:
            # 极端分支：仅在外部错误用法下触发，主流程不会走到这里
            # 中文：此处为防御性分支，正常流程不会触发
            # English: Defensive branch, not triggered in normal flow
            raise ValueError(f"Unknown server: {server_name}")  # pragma: no cover

        if config.disabled:
            raise RuntimeError(f"Cannot start disabled server: {server_name}")

        if server_name in self._active_clients:
            return  # 已经启动

        # 根据配置类型创建客户端
        client = client_factory(config, message_handler=self._message_handler)
        await client.aconnect()
        self._active_clients[server_name] = client
        try:
            await self._arefresh_tool_mapping()
        except ToolNameDuplicatedError as e:
            await client.adisconnect()
            del self._active_clients[server_name]
            raise e

    async def astop_client(self, server_name: str) -> None:
        """停止单个服务器客户端"""
        async with self._lock:
            await self._astop_client(server_name)

    async def _astop_client(self, server_name: str) -> None:
        """停止单个服务器客户端"""
        client = self._active_clients.pop(server_name, None)
        if client:
            await client.adisconnect()
            await self._arefresh_tool_mapping()

    async def _astop_all(self) -> None:
        """停止所有客户端"""
        for name in list(self._active_clients.keys()):
            await self._astop_client(name)

    async def astop_all(self) -> None:
        """停止所有客户端"""
        async with self._lock:
            logger.debug(f"Manager Stop all async task: {asyncio.current_task()}")
            await self._astop_all()

    def _clear_all(self) -> None:
        """清空所有连接（别名）"""
        self._servers_config.clear()
        self._active_clients.clear()
        self._tool_mapping.clear()
        self._alias_mapping.clear()
        self._disabled_tools.clear()

    async def aclose(self) -> None:
        """关闭所有连接（别名）"""
        await self.astop_all()

        # 2. 清空所有状态存储
        self._clear_all()

    async def _arefresh_tool_mapping(self) -> None:
        """刷新工具映射和禁用状态"""
        # 清空现有映射
        self._tool_mapping.clear()
        self._disabled_tools.clear()
        self._alias_mapping.clear()

        # 临时存储工具源服务器
        tool_sources: dict[TOOL_NAME, list[str]] = defaultdict(list)

        # 收集所有活动服务器的工具
        for server_name, client in self._active_clients.items():
            config = self._servers_config[server_name]
            try:
                tools = await client.list_tools()
                for t in tools:
                    original_tool_name = t.name
                    # 获取合并后的工具元数据（浅合并，具体配置优先，其次使用默认配置）
                    # Get merged tool meta (shallow merge: specific overrides default)
                    tool_meta = self._merged_tool_meta(config, original_tool_name)

                    # 确定最终显示的工具名（优先使用别名）
                    display_name: str = tool_meta.alias if tool_meta and tool_meta.alias else original_tool_name
                    # 如果使用提别名，则更新别名映射
                    if display_name != original_tool_name:
                        self._alias_mapping[display_name] = (server_name, original_tool_name)

                    # 将工具添加到映射
                    tool_sources[display_name].append(server_name)

                    # 检查是否为禁用工具 (根据配置，但此时需要注意如果原始名称在禁用列表中，也应该禁用，因为此处的禁用列表是归属于某个
                    # ServerConfig的，不存在重复名称的情况，用户有可能配置了alias，但是使用原始名称禁用。)
                    if display_name in (config.forbidden_tools or []) or original_tool_name in (config.forbidden_tools or []):
                        self._disabled_tools.add(display_name)
            except Exception as e:
                logger.error(f"Error listing tools for {server_name}: {e}")

        # 构建最终映射（处理工具名冲突）
        for tool, sources in tool_sources.items():
            if len(sources) > 1:
                logger.warning(f"Warning: Tool '{tool}' exists in multiple servers: {sources}")
                suggestion = (
                    "Please use the 'alias' feature in ToolMeta to resolve conflicts. "
                    "Each tool should have a unique name or alias across all servers."
                )
                raise ToolNameDuplicatedError(f"Tool '{tool}' exists in multiple servers: {sources}\n{suggestion}")
            self._tool_mapping[tool] = sources[0]

    async def avalidate_tool_call(self, tool_name: TOOL_NAME, parameters: dict) -> tuple[SERVER_NAME, TOOL_NAME]:
        """
        判断工具调用的合法性，如果合法，返回对应的服务名称与原始工具名称

        Args:
            tool_name (str): 被调用的工具名称，可能是alias
            parameters (dict): 工具调用的参数

        Returns:
            tuple[SERVER_NAME, TOOL_NAME]: 经过校验后的合法服务名与工具名
        """
        # 标记当前parameters尚未被使用
        logger.debug(f"{parameters}未被检查。当前版本不支持Schema校验。")
        # 检查工具是否可用
        if tool_name in self._disabled_tools:
            raise PermissionError(f"Tool '{tool_name}' is disabled by configuration")

        server_name = self._tool_mapping.get(tool_name)
        if not server_name:
            raise ValueError(f"Tool '{tool_name}' not found in any active server")

        # 如果tool_name是一个别名，则使用别名映射到原始名称
        if tool_name in self._alias_mapping:
            original_server_name, tool_name = self._alias_mapping[tool_name]
            assert original_server_name == server_name, "Alias mapping should map to the same server"
        return server_name, tool_name

    async def acall_tool(
        self,
        server_name: SERVER_NAME,
        tool_name: TOOL_NAME,
        parameters: dict,
        timeout: float | None = None,
    ) -> CallToolResult:
        """
        触发MCP工具的调用。注意此方法tool_name必须是工具原始名称，如果是alias别名调用，需要使用 aexecute_tool

        Args:
            server_name (str): 服务名称
            tool_name (str): 工具名称
            parameters (dict): 工具调用参数
            timeout (float | None): 超时时间

        Returns:
            CallToolResult: MCP 标准返回格式
        """
        # 获取MCP服务客户端连接
        client = self._active_clients.get(server_name)
        if not client:
            raise RuntimeError(f"Server '{server_name}' for tool '{tool_name}' is not active")

        # 获取合并后的工具元数据
        config = self._servers_config[server_name]
        tool_meta = self._merged_tool_meta(config, tool_name)

        # 执行工具调用
        try:
            if timeout:
                result = await asyncio.wait_for(client.call_tool(tool_name, parameters), timeout)
            else:
                result = await client.call_tool(tool_name, parameters)

            # 如果有自定义元数据，则利用MCP协议返回Result中的meta元数据携带能力透传。
            if tool_meta:
                if result.meta:
                    result.meta.setdefault(A2C_TOOL_META, {}).update(tool_meta)
                else:
                    result.meta = {A2C_TOOL_META: tool_meta}

            # 中文: 如果配置了VRL脚本,尝试对返回值进行转换
            # English: If VRL script is configured, try to transform the return value
            if config.vrl:
                try:
                    # 中文: 将CallToolResult序列化为字典，并注入tool_name和parameters作为VRL的Event输入
                    # English: Serialize CallToolResult to dict and inject tool_name and parameters as VRL Event input
                    event = result.model_dump(mode="json")
                    # 中文: 注入工具调用的上下文信息
                    # English: Inject tool call context information
                    event["tool_name"] = tool_name
                    event["parameters"] = parameters

                    # 中文: 执行VRL转换（使用系统本地时区）
                    # English: Execute VRL transformation (use system local timezone)
                    # 获取系统时区名称，例如 "Asia/Shanghai" 或 "America/New_York"
                    # Get system timezone name, e.g., "Asia/Shanghai" or "America/New_York"
                    # VRL需要IANA时区名称，尝试从tzlocal获取；若失败则使用UTC
                    # VRL requires IANA timezone name; try to get from tzlocal, fallback to UTC
                    try:
                        import tzlocal

                        timezone_name = str(tzlocal.get_localzone())
                    except Exception:
                        # 如果无法获取本地时区，回退到UTC / Fallback to UTC if local timezone unavailable
                        timezone_name = "UTC"

                    vrl_result = VRLRuntime.run(config.vrl, event, timezone=timezone_name)
                    transformed_event = vrl_result.processed_event

                    # 中文: 将转换后的结果压缩为JSON字符串存入Meta（因为Meta要求简单数据结构）
                    # English: Compress transformed result to JSON string for Meta (Meta requires simple data structure)
                    if result.meta is None:
                        result.meta = {}
                    result.meta[A2C_VRL_TRANSFORMED] = json.dumps(transformed_event, ensure_ascii=False)

                    logger.debug(f"VRL转换成功 / VRL transformation succeeded for tool '{tool_name}'")
                except Exception as e:
                    # 中文: VRL转换失败不影响正常返回，仅记录警告日志
                    # English: VRL transformation failure doesn't affect normal return, just log warning
                    logger.warning(
                        f"VRL转换失败 / VRL transformation failed for tool '{tool_name}': {e}. "
                        f"原始结果将正常返回 / Original result will be returned normally.",
                    )

            return result
        except TimeoutError:
            raise TimeoutError(f"Tool '{tool_name}' execution timed out") from None
        except Exception as e:
            raise RuntimeError(f"Tool execution failed: {e}") from e

    async def aexecute_tool(self, tool_name: TOOL_NAME, parameters: dict, timeout: float | None = None) -> CallToolResult:
        """执行指定工具 与 acall_tool 的区别在于此方法支持使用alias别名进行调用。"""
        server_name, tool_name = await self.avalidate_tool_call(tool_name, parameters)
        return await self.acall_tool(server_name, tool_name, parameters, timeout)

    def get_server_status(self) -> list[tuple[str, bool, str]]:
        """获取服务器状态列表"""
        return [
            (
                server_name,
                server_name in self._active_clients,
                "pending" if server_name not in self._active_clients else self._active_clients[server_name].state,
            )
            for server_name in self._servers_config
        ]

    async def available_tools(self) -> AsyncGenerator[Tool, Any]:
        """获取可用工具及其元数据"""
        async with self._lock:
            servers_cached_tools = defaultdict(list)
            for tool_name, server in self._tool_mapping.items():
                if server not in servers_cached_tools and server in self._active_clients:
                    client = self._active_clients[server]
                    tools = await client.list_tools()
                    servers_cached_tools[server] = tools

                config = self._servers_config[server]
                assert not config.disabled, "Server should not be disabled"

                original_server, original_tool_name = self._alias_mapping.get(tool_name) or (server, tool_name)
                assert original_server == server, "Alias mapping error"

                tool = next((t for t in tools if t.name == original_tool_name), None)
                if tool:
                    a2c_meta = self._merged_tool_meta(config, original_tool_name)
                    if a2c_meta:
                        if tool.meta is None:
                            tool.meta = {A2C_TOOL_META: a2c_meta}
                        else:
                            tool.meta.update({A2C_TOOL_META: a2c_meta})
                    yield tool

    async def list_windows(self, window_uri: str | None = None) -> list[tuple[SERVER_NAME, Resource]]:
        """
        列出所有活动MCP服务器的窗口资源，并附带其归属的server名称。
        List window resources from all active MCP servers with owning server name.

        Args:
            window_uri (str | None): 若提供，则仅返回URI完全匹配的窗口；否则返回所有窗口。

        Returns:
            list[tuple[SERVER_NAME, Resource]]: [(server_name, resource), ...]
        """
        results: list[tuple[SERVER_NAME, Resource]] = []
        # 不加锁读取活跃客户端快照，避免长时间持锁阻塞 I/O
        active_snapshot = list(self._active_clients.items())
        for server_name, client in active_snapshot:
            try:
                resources = await client.list_windows()
            except Exception as e:
                logger.error(f"Error listing windows for {server_name}: {e}")
                continue

            for res in resources:
                if window_uri is not None and str(res.uri) != window_uri:
                    continue
                results.append((server_name, res))
        return results

    async def get_windows_details(self, window_uri: str | None = None) -> list[tuple[SERVER_NAME, Resource, ReadResourceResult]]:
        """
        中文: 读取所有活动 MCP 服务器的窗口资源详情。由于 MCP 协议中的 Resource 仅为标识，需要通过 read_resource 获取内容。
        英文: Read detailed contents for window resources from all active MCP servers. Resource is an identifier; use read_resource.

        Args:
            window_uri (str | None): 若提供，则仅读取该 URI 完全匹配的窗口；否则读取所有窗口。

        Returns:
            list[tuple[SERVER_NAME, Resource, list[object]]]: 列表项为 (server_name, resource, contents)。
        """
        details: list[tuple[SERVER_NAME, Resource, ReadResourceResult]] = []
        active_snapshot = list(self._active_clients.items())
        for server_name, client in active_snapshot:
            try:
                resources = await client.list_windows()
            except Exception as e:
                logger.error(f"Error listing windows for {server_name}: {e}")
                continue

            for res in resources:
                if window_uri is not None and str(res.uri) != window_uri:
                    continue
                content = await client.get_window_detail(res)
                details.append((server_name, res, content))
        return details

    @staticmethod
    def _merged_tool_meta(config: MCPServerConfig, tool_name: TOOL_NAME) -> ToolMeta | None:
        """
        浅层合并工具元数据：优先使用具体 tool_meta，若字段缺失则回落到 default_tool_meta。
        Shallow merge ToolMeta: prefer per-tool meta; fallback to default for missing root-level fields.
        """
        specific = (config.tool_meta or {}).get(tool_name)
        default = config.default_tool_meta
        if specific is None and default is None:
            return None
        if specific is None:
            return default
        if default is None:
            return specific
        # 仅根级字段浅合并；specific优先
        merged: dict = {}
        # Pydantic v2: model_dump 可排除 None，以避免用 None 覆盖
        merged.update(default.model_dump(exclude_none=True))
        merged.update(specific.model_dump(exclude_none=True))
        return ToolMeta(**merged)



# -*- coding: utf-8 -*-
# filename: test_computer_exceptions.py
from __future__ import annotations

import pytest
from mcp import StdioServerParameters

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.mcp_clients.model import StdioServerConfig


@pytest.mark.asyncio
async def test_arender_and_validate_server_missing_input_keeps_original() -> None:
    """
    当出现未定义的 ${input:NOT_DEFINED} 时，渲染器按设计返回原值字符串而非抛错，
    因此应成功返回校验后的模型，且 env 中仍保留占位符字符串。
    覆盖 computer.py 中对未定义输入的 warning 日志路径。
    """
    comp = Computer(name="test", inputs=[], mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    params = StdioServerParameters(
        command="echo",
        args=["hello"],
        env={"FOO": "${input:NOT_DEFINED}"},
        cwd=None,
        encoding="utf-8",
        encoding_error_handler="strict",
    )
    cfg = StdioServerConfig(name="bad", server_parameters=params)

    validated = await comp._arender_and_validate_server(cfg)  # type: ignore[attr-defined]
    assert validated.server_parameters.env is not None
    assert validated.server_parameters.env.get("FOO") == "${input:NOT_DEFINED}"


@pytest.mark.asyncio
async def test_boot_up_render_error_keeps_original_config(monkeypatch: pytest.MonkeyPatch) -> None:
    """
    让渲染器抛出异常，覆盖 boot_up 中的异常分支（115-118），应当保留原配置继续初始化。
    """
    comp = Computer(name="test", inputs=[], mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    params = StdioServerParameters(
        command="echo",
        args=["hello"],
        env=None,
        cwd=None,
        encoding="utf-8",
        encoding_error_handler="strict",
    )
    cfg = StdioServerConfig(name="keep", server_parameters=params)

    # 往内部 mcp_servers 放一个配置
    comp._mcp_servers = {cfg}  # type: ignore[attr-defined]

    # 让 arender 抛出异常
    async def boom(*args, **kwargs):  # noqa: ANN001, ANN003
        raise RuntimeError("render failed")

    comp._config_render.arender = boom  # type: ignore[assignment]

    await comp.boot_up()
    # 成功则说明吞掉异常并保留了原配置，manager 初始化完毕
    assert comp.mcp_manager is not None



# -*- coding: utf-8 -*-
# filename: test_sse_client.py
# @Time    : 2025/8/19
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from unittest.mock import AsyncMock, MagicMock, patch

import pytest
from mcp.client.session_group import SseServerParameters
from polyfactory.factories.pydantic_factory import ModelFactory

from a2c_smcp.computer.mcp_clients.sse_client import SseMCPClient


@pytest.mark.asyncio
async def test_abefore_connect_and_on_enter_connected():
    """
    测试 abefore_connect 与 on_enter_connected 的协作，确保 EventData.kwargs 能正确传递 client_session。
    Test abefore_connect and on_enter_connected, ensuring EventData.kwargs passes client_session correctly.
    """
    # 构造 mock 对象 Construct mock objects
    mock_aread_stream = MagicMock(name="aread_stream")
    mock_awrite_stream = MagicMock(name="awrite_stream")
    mock_session = AsyncMock(name="ClientSession")
    mock_initialize = AsyncMock()
    mock_session.initialize = mock_initialize

    mock_params = ModelFactory.create_factory(model=SseServerParameters).build()

    # Patch sse_client 和 ClientSession
    with (
        patch("a2c_smcp.computer.mcp_clients.sse_client.sse_client", new=AsyncMock(return_value=(mock_aread_stream, mock_awrite_stream))),
        patch("a2c_smcp.computer.mcp_clients.sse_client.ClientSession", new=AsyncMock(return_value=mock_session)) as mock_cs,
    ):
        # 构造 client 和 event
        client = SseMCPClient(params=mock_params)
        assert client._async_session is None
        client._aexit_stack = AsyncMock()  # 避免真实上下文
        client._aexit_stack.enter_async_context = AsyncMock(side_effect=[(mock_aread_stream, mock_awrite_stream), mock_session])
        # 调用 abefore_connect
        await client.aconnect()
        # 断言 ClientSession 被正确初始化
        mock_cs.assert_called_once_with(mock_aread_stream, mock_awrite_stream, message_handler=None)
        # 断言 initialize 被调用
        mock_initialize.assert_awaited()
        # 断言 client._async_session 设置正确
        assert client._async_session is mock_session
        assert client._async_session is not None



# -*- coding: utf-8 -*-
# filename: resources_subscribe_b_stdio_server.py
# @Time    : 2025/10/02 20:22
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: 提供 Resources + Subscribe + Tool 的 MCP Stdio 服务器（B 版本），用于 e2e 测试多服务窗口与调用顺序。
英文: MCP Stdio server exposing Resources + Subscribe + Tool (variant B) for e2e tests.
"""

from __future__ import annotations

import anyio
import mcp.types as types
from mcp.server.lowlevel.server import Server as LowLevelServer
from mcp.server.stdio import stdio_server


class TestServer(LowLevelServer):
    def get_capabilities(self, notification_options, experimental_capabilities) -> types.ServerCapabilities:  # type: ignore[override]
        caps = super().get_capabilities(notification_options, experimental_capabilities)
        if types.SubscribeRequest in self.request_handlers or types.UnsubscribeRequest in self.request_handlers:
            resources_cap = caps.resources or types.ResourcesCapability(subscribe=False, listChanged=False)
            caps = types.ServerCapabilities(
                prompts=caps.prompts,
                resources=types.ResourcesCapability(subscribe=True, listChanged=resources_cap.listChanged),
                tools=caps.tools,
                logging=caps.logging,
                experimental=caps.experimental,
                completions=caps.completions,
            )
        return caps


WINDOW_HOST = "example.desktop.subscribe.b"
WINDOW_RESOURCES: list[types.Resource] = [
    types.Resource(
        uri=f"window://{WINDOW_HOST}/main?priority=50",
        name="Main-B",
        description="中文: B 主窗口; 英文: B main window",
        mimeType="text/markdown",
    ),
    types.Resource(
        uri=f"window://{WINDOW_HOST}/board?priority=85",
        name="Board-B",
        description="中文: B 看板; 英文: B board",
        mimeType="text/markdown",
    ),
]


async def run() -> None:
    server = TestServer(name="itest-resources-subscribe-b", version="0.0.1", instructions="itest-desktop-b")

    @server.list_resources()
    async def list_resources(req: types.ListResourcesRequest | None = None) -> list[types.Resource]:  # noqa: ARG001
        return WINDOW_RESOURCES

    @server.read_resource()
    async def read_resource(uri: types.AnyUrl):  # noqa: ARG001
        return "# B Window Resource\n\n中文: B 版本窗口; 英文: B variant window.\n"

    # 工具：mark_b
    @server.list_tools()
    async def list_tools() -> list[types.Tool]:
        return [
            types.Tool(
                name="mark_b",
                description="中文: 标记B; 英文: mark B",
                inputSchema={"type": "object", "properties": {}},
            ),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: dict | None):  # noqa: ARG001
        return [types.TextContent(type="text", text=f"ok:{name}")]

    @server.subscribe_resource()
    async def subscribe(uri: types.AnyUrl):  # noqa: ARG001
        ctx = server.request_context
        await ctx.session.send_resource_updated(uri)

    @server.unsubscribe_resource()
    async def unsubscribe(_: types.AnyUrl):  # noqa: ARG001
        return None

    async with stdio_server() as (read_stream, write_stream):
        init_opts = server.create_initialization_options()
        await server.run(read_stream, write_stream, init_opts)


if __name__ == "__main__":
    anyio.run(run)



# -*- coding: utf-8 -*-
# filename: test_socketio_client_desktop.py
# @Time    : 2025/10/05 13:32
# @Author  : A2C-SMCP
# @Email   : qa@a2c-smcp.local
# @Software: PyTest
"""
中文：Computer Socket.IO 客户端的桌面相关单元测试。
English: Unit tests for Computer Socket.IO client's desktop features.
"""

from typing import Any

import pytest

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.socketio.client import SMCPComputerClient
from a2c_smcp.smcp import SMCP_NAMESPACE, UPDATE_DESKTOP_EVENT, GetDeskTopReq, GetDeskTopRet


class _DummyComputer(Computer):
    async def get_desktop(self, size: int | None = None, window_uri: str | None = None) -> list[str]:  # type: ignore[override]
        return ["window://dummy\n\ncontent"]


@pytest.mark.asyncio
async def test_on_get_desktop_returns_expected(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = _DummyComputer(name="comp-sid-1", auto_connect=False, auto_reconnect=False)
    client = SMCPComputerClient(computer=comp)
    # 准备 office 情况
    client.office_id = "office-1"

    req: GetDeskTopReq = {
        "computer": "comp-sid-1",
        "agent": "office-1",
        "req_id": "rid-1",
        "desktop_size": 1,
    }

    ret: GetDeskTopRet = await client.on_get_desktop(req)
    assert ret["req_id"] == "rid-1"
    assert isinstance(ret["desktops"], list) and ret["desktops"]


@pytest.mark.asyncio
async def test_emit_refresh_desktop_emits(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = _DummyComputer(name="test", auto_connect=False, auto_reconnect=False)
    client = SMCPComputerClient(computer=comp)
    client.office_id = "office-1"

    called: dict[str, Any] = {}

    async def fake_emit(self, event: str, data: Any = None, namespace: str | None = None, callback: Any = None) -> None:  # noqa: D401
        called["event"] = event
        called["data"] = data

    monkeypatch.setattr(SMCPComputerClient, "emit", fake_emit)

    await client.emit_refresh_desktop()
    assert called["event"] == UPDATE_DESKTOP_EVENT
    assert called["data"]["computer"] == "test"



"""
* 文件名: auth
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: None
* 描述: 认证接口抽象定义 / Authentication interface abstract definition
"""

from abc import ABC, abstractmethod

from socketio import AsyncServer


class AuthenticationProvider(ABC):
    """
    认证提供者抽象基类，用于处理Socket.IO连接的认证逻辑
    Abstract base class for authentication providers, handles Socket.IO connection authentication logic
    """

    @abstractmethod
    async def authenticate(self, sio: AsyncServer, environ: dict, auth: dict | None, headers: list) -> bool:
        """
        认证连接请求
        Authenticate connection request

        Args:
            sio (AsyncServer): Socket.IO服务器实例 / Socket.IO server instance
            environ (dict): 请求环境变量 / Request environment variables
            auth (dict | None): 原始认证数据 / Raw authentication data
            headers (list): 原始请求头列表 / Raw request headers list

        Returns:
            bool: 认证是否成功 / Whether authentication succeeded
        """
        pass


class DefaultAuthenticationProvider(AuthenticationProvider):
    """
    默认认证提供者，提供基础的认证逻辑实现
    Default authentication provider, provides basic authentication logic implementation
    """

    def __init__(self, admin_secret: str | None = None, api_key_name: str = "x-api-key") -> None:
        """
        初始化默认认证提供者
        Initialize default authentication provider

        Args:
            admin_secret (str | None): 管理员密钥 / Admin secret
            api_key_name (str): API密钥字段名 / API key field name
        """
        self.admin_secret = admin_secret
        self.api_key_name = api_key_name

    async def authenticate(self, sio: AsyncServer, environ: dict, auth: dict | None, headers: list) -> bool:
        """
        默认认证逻辑：从headers中提取API密钥进行认证
        Default authentication logic: extract API key from headers for authentication
        """
        # 从headers中提取API密钥
        # Extract API key from headers
        api_key = None
        for header in headers:
            if isinstance(header, (list, tuple)) and len(header) >= 2:
                header_name = header[0].decode("utf-8").lower() if isinstance(header[0], bytes) else str(header[0]).lower()
                header_value = header[1].decode("utf-8") if isinstance(header[1], bytes) else str(header[1])

                if header_name == self.api_key_name.lower():
                    api_key = header_value
                    break

        if not api_key:
            return False

        # 检查管理员权限：与配置的管理员密钥比较
        # Check admin permission: compare with configured admin secret
        if self.admin_secret is not None and api_key == self.admin_secret:
            return True

        # 这里可以添加其他认证逻辑，如数据库验证等
        # Additional authentication logic can be added here, such as database validation
        return False



# -*- coding: utf-8 -*-
# filename: test_vrl_integration.py
# @Time    : 2025/10/6 14:30
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm

"""
VRL集成测试 / VRL Integration Tests

测试VRL脚本在MCP Server配置中的语法检查和工具返回值转换功能。
Tests VRL script syntax validation and tool return value transformation in MCP Server config.
"""

import json

import pytest
from mcp import StdioServerParameters
from mcp.types import CallToolResult, TextContent
from polyfactory.factories.pydantic_factory import ModelFactory

from a2c_smcp.computer.mcp_clients.model import A2C_VRL_TRANSFORMED, StdioServerConfig


def test_vrl_syntax_validation_success():
    """测试VRL语法验证成功 / Test VRL syntax validation success"""
    # 有效的VRL脚本
    # Valid VRL script
    valid_vrl = '.result = "success"'

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=valid_vrl,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == valid_vrl


def test_vrl_syntax_validation_failure():
    """测试VRL语法验证失败 / Test VRL syntax validation failure"""
    # 无效的VRL脚本
    # Invalid VRL script
    invalid_vrl = "this is not valid vrl syntax!!!"

    with pytest.raises(ValueError, match="VRL语法错误"):
        StdioServerConfig(
            name="test_server",
            disabled=False,
            forbidden_tools=[],
            tool_meta={},
            vrl=invalid_vrl,
            server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
        )


def test_vrl_none_or_empty_allowed():
    """测试VRL字段允许为None或空字符串 / Test VRL field allows None or empty string"""
    # None值应该被接受
    # None should be accepted
    config1 = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=None,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )
    assert config1.vrl is None

    # 空字符串应该被接受
    # Empty string should be accepted
    config2 = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl="",
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )
    assert config2.vrl == ""


def test_vrl_transformation_result_format():
    """测试VRL转换结果的存储格式 / Test VRL transformation result storage format"""
    # 模拟一个CallToolResult
    # Mock a CallToolResult
    result = CallToolResult(
        content=[TextContent(text="test result", type="text")],
        isError=False,
    )

    # 模拟VRL转换后的数据
    # Mock VRL transformed data
    transformed_data = {
        "status": "success",
        "message": "transformed",
        "data": {"key": "value"},
    }

    # 按照实现逻辑，转换后的数据应该被JSON序列化存储
    # According to implementation, transformed data should be JSON serialized
    result.meta = {A2C_VRL_TRANSFORMED: json.dumps(transformed_data, ensure_ascii=False)}

    # 验证存储格式
    # Verify storage format
    assert A2C_VRL_TRANSFORMED in result.meta
    assert isinstance(result.meta[A2C_VRL_TRANSFORMED], str)

    # 验证可以反序列化
    # Verify it can be deserialized
    restored = json.loads(result.meta[A2C_VRL_TRANSFORMED])
    assert restored == transformed_data


def test_vrl_complex_script():
    """测试复杂的VRL脚本 / Test complex VRL script"""
    # 一个更复杂的VRL脚本，包含多个操作
    # A more complex VRL script with multiple operations
    complex_vrl = """
    .status = "processed"
    .timestamp = now()
    .transformed = true
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=complex_vrl,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == complex_vrl


def test_vrl_with_conditional_logic():
    """测试带条件逻辑的VRL脚本 / Test VRL script with conditional logic"""
    # VRL支持条件逻辑
    # VRL supports conditional logic
    conditional_vrl = """
    if .isError == true {
        .status = "error"
    } else {
        .status = "success"
    }
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=conditional_vrl,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == conditional_vrl


def test_vrl_with_tool_name_access():
    """测试VRL脚本可以访问tool_name字段 / Test VRL script can access tool_name field"""
    # 中文: VRL脚本可以访问注入的tool_name字段
    # English: VRL script can access injected tool_name field
    vrl_script = """
    .tool_info = .tool_name
    .is_search_tool = .tool_name == "search"
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == vrl_script


def test_vrl_with_parameters_access():
    """测试VRL脚本可以访问parameters字段 / Test VRL script can access parameters field"""
    # 中文: VRL脚本可以访问注入的parameters字段
    # English: VRL script can access injected parameters field
    vrl_script = """
    .query = .parameters.query
    .limit = .parameters.limit
    .has_filter = exists(.parameters.filter)
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == vrl_script


def test_vrl_conditional_based_on_tool_name():
    """测试基于tool_name的条件逻辑 / Test conditional logic based on tool_name"""
    # 中文: VRL脚本可以根据tool_name执行不同的转换逻辑
    # English: VRL script can execute different transformation logic based on tool_name
    vrl_script = """
    if .tool_name == "search" {
        .result_type = "search_result"
        .query = .parameters.query
    } else if .tool_name == "execute" {
        .result_type = "execution_result"
        .command = .parameters.cmd
    } else {
        .result_type = "unknown"
    }
    .processed = true
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == vrl_script


def test_vrl_parameters_nested_access():
    """测试VRL脚本访问嵌套的parameters字段 / Test VRL script accessing nested parameters fields"""
    # 中文: VRL脚本可以访问parameters中的嵌套字段
    # English: VRL script can access nested fields in parameters
    vrl_script = """
    .user_id = .parameters.user.id
    .user_name = .parameters.user.name
    .options_enabled = .parameters.options.enabled
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == vrl_script


def test_vrl_combine_result_and_context():
    """测试VRL脚本同时处理结果和上下文信息 / Test VRL script processing both result and context"""
    # 中文: VRL脚本可以同时访问工具返回结果和调用上下文
    # English: VRL script can access both tool result and call context
    vrl_script = """
    .summary = {
        "tool": .tool_name,
        "params": .parameters,
        "is_error": to_bool(.isError) ?? false,
        "content_count": length!(.content)
    }
    """

    config = StdioServerConfig(
        name="test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )

    assert config.vrl == vrl_script



"""
文件名: conftest.py
作者: JQQ
创建日期: 2025/9/22
最后修改日期: 2025/9/22
版权: 2023 JQQ. All rights reserved.
依赖: pytest, pexpect
描述:
  中文: e2e 测试公共夹具，负责启动与关闭 CLI 交互进程。
  English: Common fixtures for e2e tests to spawn and cleanup CLI interactive process.
"""

from __future__ import annotations

import os
import shutil
import signal
import sys
from collections.abc import Iterator
from contextlib import contextmanager

import pytest

from tests.e2e.computer.utils import PROMPT_RE, expect_prompt_stable

pexpect = pytest.importorskip("pexpect", reason="e2e tests require pexpect; install with `pip install pexpect`.")


@contextmanager
def _spawn_cli(*extra_args: str, cwd: str | None = None) -> Iterator[pexpect.spawn]:
    """
    中文: 启动 CLI 交互进程，返回 pexpect child；确保在退出时清理。可通过 `cwd` 指定子进程工作目录，默认使用项目根目录。
    English: Spawn the CLI interactive process and ensure cleanup on exit. You can specify child process working directory via `cwd`;
        defaults to project root.
    """
    print("spawn cli...")
    env = os.environ.copy()
    # 确保 Python 输出不被缓冲，便于 pexpect 捕获 / Unbuffered Python output for stable pexpect reads
    env["PYTHONUNBUFFERED"] = "1"
    env["PYTHONIOENCODING"] = "utf-8"
    # 降低 prompt_toolkit 的控制序列噪音（如 CPR），提升匹配稳定性
    # Reduce prompt_toolkit control sequences to stabilize matching
    env["PROMPT_TOOLKIT_NO_CPR"] = "1"
    env["PROMPT_TOOLKIT_DISABLE_BRACKETED_PASTE"] = "1"
    env["PROMPT_TOOLKIT_COMPLETE_STYLE"] = "column"
    env["PROMPT_TOOLKIT_EDITING_MODE"] = "emacs"
    env["PROMPT_TOOLKIT_MOUSE_SUPPORT"] = "0"
    env["PROMPT_TOOLKIT_ENABLE_SUSPEND"] = "0"
    # 使用最简终端，促使 prompt_toolkit 降级，减少 ANSI 控制序列
    # Use dumb TERM to reduce advanced terminal features
    env["TERM"] = "dumb"  # 使用最简单的终端类型
    # 禁用分页与固定列宽，进一步减少输出差异 / Disable pager and fix width to reduce variability
    env["PAGER"] = "cat"
    env["COLUMNS"] = "120"
    env["LINES"] = "24"
    env["NO_COLOR"] = "1"
    # 强制使用UTF-8编码 / Force UTF-8 encoding
    env["LC_ALL"] = "en_US.UTF-8"
    env["LANG"] = "en_US.UTF-8"
    # 优先使用已安装的 console script；否则回退到 python -c 调用 main()
    # Prefer console script if available; fallback to python -c main()
    console_script = shutil.which("a2c-computer")
    if console_script:
        args = [console_script, "--no-color", "run"]
    else:
        print("a2c-computer not found in shell")
        args = [
            sys.executable,
            "-c",
            "from a2c_smcp.computer.cli.main import main; main()",
            "--no-color",
            "run",
        ]
    if extra_args:
        args.extend(extra_args)

    # 计算与设置工作目录 / Compute and set working directory
    # 默认将工作目录设置为项目根目录（本文件位于 tests/e2e/conftest.py，向上两级即为项目根）
    # By default, set cwd to project root (this file lives at tests/e2e/conftest.py; go up two levels)
    project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", ".."))
    spawn_cwd = cwd or project_root

    print("a2c-computer starting...")
    # 保证每次发送前有一个时延保持稳定
    child = pexpect.spawn(args[0], args[1:], env=env, encoding="utf-8", timeout=60, cwd=spawn_cwd)
    # 控制窗口大小，减少 CPR 请求 / Set winsize to reduce CPR
    child.delaybeforesend = 0.1
    try:
        child.setwinsize(24, 120)
    except Exception:
        pass
    try:
        yield child
    finally:
        # 优雅退出; 若仍存活则强杀 / Try graceful exit then hard kill if needed
        if child.isalive():
            try:
                child.sendline("exit")
                child.expect([pexpect.EOF, "Bye"], timeout=5)
            except Exception:
                pass
        if child.isalive():
            try:
                child.kill(signal.SIGKILL)
            except Exception:
                pass


@pytest.fixture()
def cli_proc() -> Iterator[pexpect.spawn]:
    """
    中文: 提供一个已启动并等待在 `a2c>` 提示符的 CLI 进程。
    English: Provide a CLI process ready at `a2c>` prompt.
    """
    with _spawn_cli() as child:
        print("a2c-computer started up")
        child.expect([r"Enter interactive mode, type 'help' for commands", PROMPT_RE])
        # 若匹配到横幅，则继续等待提示符 / If banner matched, then wait for prompt
        if (
            child.match
            and hasattr(child.match, "re")
            and child.match.re
            and getattr(child.match.re, "pattern", "").startswith("Enter interactive")
        ):
            pass  # fall through to wait prompt below
        # 等待提示符，并在必要时发送空回车触发刷新 / Wait for prompt, poke with empty enter if needed
        for _ in range(5):
            try:
                print("waiting for [a2c>]...")
                expect_prompt_stable(child, quiet=0.5, max_wait=5.0)
                break
            except pexpect.TIMEOUT:
                child.sendline("")
        else:
            child.expect(PROMPT_RE)
        child.sendline("")
        expect_prompt_stable(child, quiet=0.5, max_wait=12.0)
        yield child



# -*- coding: utf-8 -*-
# filename: test_base_client.py
# @Time    : 2025/8/18 19:26
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
import asyncio
from asyncio import Queue
from contextlib import AsyncExitStack
from unittest.mock import AsyncMock, MagicMock

import pytest
from mcp import ClientSession
from mcp.types import Implementation, InitializeResult, ListToolsResult, ServerCapabilities
from pydantic import BaseModel
from transitions.core import MachineError

from a2c_smcp.computer.mcp_clients.base_client import STATES, BaseMCPClient


# 创建测试用的具体客户端实现
class MockMCPClient(BaseMCPClient):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._async_session = None
        self.prepare_called = False
        self.can_called = False
        self.before_called = False
        self.after_called = False
        self.on_enter_called = False
        self.disabled = False
        self.forbidden_tools = []
        self.tool_meta = {}
        self.aexit_stack = AsyncMock(spec=AsyncExitStack)

    async def _create_async_session(self) -> ClientSession:
        return MagicMock(spec=ClientSession)

    async def aprepare_connect(self, event):
        await super().aprepare_connect(event)
        self.prepare_called = True

    async def acan_connect(self, event):
        await super().acan_connect(event)
        self.can_called = True
        return True

    async def abefore_connect(self, event):
        await super().abefore_connect(event)
        self.before_called = True

    async def on_enter_connected(self, event):
        await super().on_enter_connected(event)

    async def aafter_connect(self, event):
        await super().aafter_connect(event)
        self.after_called = True

    async def on_enter_disconnected(self, event):
        self.on_enter_called = True
        await super().on_enter_disconnected(event)


# ------------------------------
# InitializeResult 相关的辅助类 / Helpers for InitializeResult related tests
# ------------------------------


class DummyParams(BaseModel):
    """
    中文: 伪造的参数模型，用于构造客户端实例。
    英文: Dummy parameter model used to construct client instance.
    """

    value: int = 1


class FakeClientSession:
    """
    中文: 伪造的 ClientSession，仅实现 initialize() 用于返回固定的 InitializeResult。
    英文: Fake ClientSession implementing initialize() to return a fixed InitializeResult.
    """

    def __init__(self) -> None:
        self.initialized = False

    async def initialize(self) -> InitializeResult:
        self.initialized = True
        return InitializeResult(
            protocolVersion="2025-06-18",
            capabilities=ServerCapabilities(),
            serverInfo=Implementation(name="fake", version="1.0.0"),
            instructions="test",
        )


class InitializeResultTestClient(BaseMCPClient):
    """
    中文: 用于测试的 BaseMCPClient 子类，返回 FakeClientSession，避免与现有 TestMCPClient 命名冲突。
    英文: Test subclass of BaseMCPClient returning a FakeClientSession, avoiding name conflict with TestMCPClient.
    """

    async def _create_async_session(self):
        # 这里不做额外上下文，仅返回伪造 session / No extra context, just return fake session
        return FakeClientSession()


# pytest fixture 创建测试客户端
@pytest.fixture
async def client():
    """创建一个测试用的 MCP 客户端实例"""
    state_changes = Queue()

    # 状态变化回调收集器
    async def state_change_callback(from_state, to_state):
        await state_changes.put((from_state, to_state))

    # 创建并返回客户端和状态变化队列
    client = MockMCPClient(params=MagicMock(), state_change_callback=state_change_callback)
    return client, state_changes


@pytest.fixture
async def connected_client(client):
    """返回已连接状态的客户端"""
    client, state_changes = client
    await client.aconnect()
    return client, state_changes


# 辅助函数用于从队列中获取状态变化
async def get_next_state_change(state_changes):
    """从状态变化队列中获取下一个变化，最多等待1秒"""
    return await asyncio.wait_for(state_changes.get(), timeout=1.0)


# 测试用例
@pytest.mark.asyncio
async def test_initial_state(client):
    """测试初始状态是否正确"""
    client_instance, _ = client
    assert client_instance.state == STATES.initialized


@pytest.mark.asyncio
async def test_successful_connection_flow(client):
    """测试完整的连接流程"""
    client_instance, state_changes = client

    # 触发连接
    await client_instance.aconnect()

    # 验证状态转换
    assert client_instance.state == STATES.connected

    # 验证回调函数被调用
    from_state, to_state = await get_next_state_change(state_changes)
    assert from_state == STATES.initialized.value
    assert to_state == STATES.connected.value

    # 验证各阶段回调被调用
    assert client_instance.prepare_called
    assert client_instance.can_called
    assert client_instance.before_called
    assert client_instance.after_called


@pytest.mark.asyncio
async def test_disconnection_flow(connected_client):
    """测试断开连接流程"""
    client_instance, state_changes = connected_client
    await state_changes.get()  # 清除初始连接状态变化

    # 触发断开连接
    await client_instance.adisconnect()

    # 验证状态转换
    assert client_instance.state == STATES.disconnected

    # 验证回调函数被调用
    from_state, to_state = await get_next_state_change(state_changes)
    assert from_state == STATES.connected
    assert to_state == STATES.disconnected

    # 验证断开时的资源清理
    assert client_instance._async_session is None
    assert client_instance.on_enter_called


@pytest.mark.asyncio
async def test_disconnect_from_disconnected_state(connected_client):
    """测试从断开状态断开连接"""
    client_instance, _ = connected_client
    await client_instance.adisconnect()
    assert client_instance.state == STATES.disconnected
    with pytest.raises(MachineError):
        await client_instance.adisconnect()


@pytest.mark.asyncio
async def test_error_transition(client):
    """测试错误状态转换"""
    client_instance, state_changes = client

    # 触发错误
    await client_instance.aerror()

    # 验证状态转换
    assert client_instance.state == STATES.error

    # 验证回调函数被调用
    from_state, to_state = await get_next_state_change(state_changes)
    assert from_state == STATES.initialized
    assert to_state == STATES.error


@pytest.mark.asyncio
async def test_async_session_property(client):
    """测试async_session属性"""
    client_instance, _ = client

    # 初始状态下会话为空
    assert client_instance._async_session is None

    # 访问属性触发连接
    session = await client_instance.async_session

    # 验证会话已创建
    assert session is not None

    # 验证状态转换
    assert client_instance.state == STATES.connected


@pytest.mark.asyncio
async def test_list_tools_success(connected_client):
    """测试list_tools成功场景"""
    client_instance, _ = connected_client
    # 模拟会话对象
    mock_session = client_instance._async_session
    ret_mock = MagicMock(spec=ListToolsResult)
    ret_mock.tools = []
    ret_mock.nextCursor = None
    mock_session.list_tools = AsyncMock(return_value=ret_mock)

    # 调用方法
    await client_instance.list_tools()

    # 验证正确调用
    mock_session.list_tools.assert_called()


@pytest.mark.asyncio
async def test_list_tools_error(client):
    """测试list_tools在未连接时抛出异常"""
    client_instance, _ = client
    with pytest.raises(ConnectionError, match="Not connected to server"):
        await client_instance.list_tools()


@pytest.mark.asyncio
async def test_call_tool_success(connected_client):
    """测试call_tool成功场景"""
    client_instance, _ = connected_client
    # 模拟会话对象
    mock_session = client_instance._async_session
    mock_session.call_tool = AsyncMock()

    # 调用方法
    await client_instance.call_tool("test_tool", {"param": "value"})

    # 验证正确调用
    mock_session.call_tool.assert_called_with("test_tool", {"param": "value"})


@pytest.mark.asyncio
async def test_call_tool_error(client):
    """测试call_tool在未连接时抛出异常"""
    client_instance, _ = client
    with pytest.raises(ConnectionError, match="Not connected to server"):
        await client_instance.call_tool("test_tool", {})


@pytest.mark.asyncio
async def test_failed_connection(client, mocker):
    """测试连接失败场景"""
    client_instance, state_changes = client

    # 模拟连接条件检查失败
    mocker.patch.object(MockMCPClient, "acan_connect", AsyncMock(return_value=False))

    # 触发连接
    await client_instance.aconnect()

    # 验证状态未变化
    assert client_instance.state == STATES.initialized

    # 验证没有状态变化回调
    assert state_changes.empty()


@pytest.mark.asyncio
async def test_transition_to_initialized(connected_client):
    """测试初始化状态转换"""
    client_instance, state_changes = connected_client
    await state_changes.get()  # 清除初始连接状态变化

    # 触发初始化
    await client_instance.ainitialize()

    # 验证状态转换
    assert client_instance.state == STATES.initialized

    # 验证回调函数被调用
    from_state, to_state = await get_next_state_change(state_changes)
    assert from_state == STATES.connected
    assert to_state == STATES.initialized

    # 验证资源被清理
    assert client_instance._async_session is None


@pytest.mark.asyncio
async def test_enter_error_state_releases_resources(client):
    """测试进入错误状态时释放资源"""
    client_instance, state_changes = client

    # 设置会话对象存在
    mock_session = AsyncMock()
    client_instance._async_session = mock_session

    # 触发错误状态
    await client_instance.aerror()

    # 确保进入错误状态时的清理被调用
    assert client_instance._async_session is not None
    # 目前 aerror 经过优化与迭代，不再显式调用 aclose 方法，而是根据目前的任务状态动态判断调用。
    client_instance.aexit_stack.aclose.assert_not_called()
    assert client_instance.state == STATES.error


# ------------------------------
# InitializeResult 相关测试 / Tests for InitializeResult logic
# ------------------------------


@pytest.mark.asyncio
async def test_initialize_result_set_and_cleared_on_disconnect():
    """
    中文: 连接后应设置 initialize_result；断开后应清理为 None。
    英文: After connect, initialize_result should be set; after disconnect, it should be cleared to None.
    """
    client = InitializeResultTestClient(DummyParams())

    # 连接 / connect
    await client.aconnect()

    # 等待会话创建成功 / wait for session creation
    await client._create_session_success_event.wait()

    # 验证初始化结果存在 / ensure initialize_result is present
    assert client.initialize_result is not None
    assert client.initialize_result.serverInfo.name == "fake"

    # 断开 / disconnect
    await client.adisconnect()

    # 等待会话真正关闭 / wait for actual session teardown
    await client._async_session_closed_event.wait()

    # 验证初始化结果被清理 / ensure initialize_result is cleared
    assert client.initialize_result is None


@pytest.mark.asyncio
async def test_initialize_result_cleared_on_error():
    """
    中文: 出错流程中也应清理 initialize_result。
    英文: initialize_result should be cleared in error path as well.
    """
    client = InitializeResultTestClient(DummyParams())

    # 连接 / connect
    await client.aconnect()
    await client._create_session_success_event.wait()
    assert client.initialize_result is not None

    # 触发错误 / trigger error (will cancel keep-alive and teardown session)
    await client.aerror()

    # 等待会话真正关闭 / wait for actual session teardown
    await client._async_session_closed_event.wait()

    # 应被清理 / should be cleared
    assert client.initialize_result is None



# -*- coding: utf-8 -*-
# filename: test_desktop_integration.py
# @Time    : 2025/10/02 16:24
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
Desktop 集成测试占位
Integration test placeholder for Desktop pipeline

说明 / Notes:
- 由于 organize 策略尚未定稿，本文件先整体跳过；
- 等策略稳定后，将接入真实的 MCPServer mock，贯通 BaseClient -> Manager -> Computer -> organize。
"""

import pytest


@pytest.mark.skip(reason="organize 策略未稳定，集成测试暂时跳过 / organizing policy not stable yet")
class TestDesktopIntegration:
    def test_placeholder(self):
        assert True



# -*- coding: utf-8 -*-
# filename: utils.py
# @Time    : 2025/9/28 12:54
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from __future__ import annotations

import re
import time

import pexpect

# 中文: ANSI 控制序列匹配与去除工具，避免 prompt_toolkit 的控制序列影响断言
# English: ANSI control-sequence helpers to avoid prompt_toolkit artifacts breaking assertions


ANSI = r"(?:\x1b\[[0-?]*[ -/]*[@-~])*"
PROMPT_RE = re.compile(ANSI + r"a2c>" + ANSI)


def strip_ansi(s: str) -> str:
    return re.sub(ANSI, "", s)


def expect_prompt_stable(child: pexpect.spawn, *, quiet: float = 0.3, max_wait: float = 10.0) -> str | None:
    """
    中文: 等待直到捕获到“最后一个稳定的 a2c> 提示符”，即提示符出现后在 quiet 秒内没有任何新增输出。
          返回：该稳定提示符之前的输出（已去除 ANSI），供断言使用。
    English: Wait until we catch the "last stable a2c> prompt" – i.e., after the prompt appears, no new output arrives
             within the quiet window. Returns the output before that stable prompt (ANSI stripped) for assertions.
    """
    deadline = time.time() + max_wait
    last_out: str
    err: bool = False
    while True:
        remaining = max(0.05, deadline - time.time())
        try:
            # 等待下一次提示符 / wait for the next prompt
            child.expect(PROMPT_RE, timeout=remaining)
        except TimeoutError as e:
            print(f"timeout: {remaining}. ")
            err = True
            raise e
        finally:
            last_out = strip_ansi((child.before or "").strip())
            if err:
                print(f"output: {last_out}")
        # 在 quiet 窗口内观察是否还有新输出 / observe if any new output arrives within the quiet window
        try:
            # read_nonblocking 会消费数据；若有数据说明提示符后仍有输出，继续等待下一次提示符
            # read_nonblocking consumes data; if any is received, more output is coming after the prompt – keep waiting
            chunk = child.read_nonblocking(size=4096, timeout=quiet)
            if chunk:
                # 若 quiet 内有输出，则不是稳定提示符，继续下一轮 / not stable yet, continue
                continue
        except Exception:  # 包含 pexpect.TIMEOUT
            # 超时表示 quiet 窗口内没有任何新输出，认为稳定 / timeout => stable prompt
            return last_out or ""
        # 若时间已用尽，返回当前捕获 / deadline reached, return current capture
        if time.time() >= deadline:
            return last_out or ""



"""
* 文件名: sync_base
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: socketio, loguru
* 描述: 同步版本基础Namespace抽象类 / Synchronous Base Namespace abstract class
"""

from typing import Any

from socketio import Namespace

from a2c_smcp.server.sync_auth import SyncAuthenticationProvider
from a2c_smcp.server.types import SID
from a2c_smcp.utils.logger import logger


class SyncBaseNamespace(Namespace):
    """
    同步基础Namespace抽象类，提供通用的连接管理和认证功能
    Synchronous Base Namespace abstract class, provides common connection management and authentication
    """

    def __init__(self, namespace: str, auth_provider: SyncAuthenticationProvider) -> None:
        """
        初始化基础Namespace
        Initialize base namespace
        """
        super().__init__(namespace=namespace)
        self.auth_provider = auth_provider
        # name到sid的映射表，用于通过name查找session
        # name-to-sid mapping table for finding session by name
        self._name_to_sid_map: dict[str, SID] = {}

    def on_connect(self, sid: SID, environ: dict, auth: dict | None = None) -> bool:
        """
        客户端连接事件处理，包含认证逻辑（同步）
        Client connection event handler with authentication (sync)
        """
        try:
            logger.info(f"SocketIO Client {sid} connecting to {self.namespace}...")

            # 提取原始请求头
            # Extract raw request headers
            headers = self._extract_headers(environ)

            # 认证逻辑，直接传递原始数据给用户
            # Authentication logic, pass raw data directly to user
            is_authenticated = self.auth_provider.authenticate(self.server, environ, auth, headers)
            if not is_authenticated:
                raise ConnectionRefusedError("Authentication failed")

            logger.info(f"SocketIO Client {sid} connected successfully to {self.namespace}")
            return True
        except Exception as e:
            logger.error(f"Connection error for {sid}: {e}")
            raise ConnectionRefusedError("Invalid connection request") from e

    def on_disconnect(self, sid: SID) -> None:
        """
        客户端断开连接事件处理（同步）
        Client disconnect event handler (sync)
        """
        logger.info(f"SocketIO Client {sid} disconnecting from {self.namespace}...")

        # 清理name映射
        # Clean up name mapping
        self._unregister_name(sid)

        rooms = self.rooms(sid)
        for room in rooms:
            if room == sid:
                continue
            self.leave_room(sid, room)
        logger.info(f"SocketIO Client {sid} disconnected from {self.namespace}")

    def trigger_event(self, event: str, *args: Any) -> Any:
        """
        触发事件，重写触发逻辑，将冒号转换为下划线（同步）
        Trigger event, override logic to replace ':' with '_' (sync)
        """
        return super().trigger_event(event.replace(":", "_"), *args)

    def _register_name(self, name: str, sid: SID) -> None:
        """
        注册name到sid的映射，如果name已存在则抛出异常
        Register name-to-sid mapping, raise exception if name already exists

        Args:
            name (str): 客户端名称 / Client name
            sid (SID): 客户端连接ID / Client connection ID

        Raises:
            ValueError: 当name已被其他sid使用时 / When name is already used by another sid
        """
        if name in self._name_to_sid_map:
            existing_sid = self._name_to_sid_map[name]
            if existing_sid != sid:
                raise ValueError(f"Name '{name}' already registered by sid '{existing_sid}' in namespace {self.namespace}")
            # 如果是同一个sid重新注册，允许（幂等操作）
            # Allow re-registration by the same sid (idempotent operation)
            logger.debug(f"Name '{name}' re-registered by same sid '{sid}'")
        else:
            self._name_to_sid_map[name] = sid
            logger.debug(f"Registered name '{name}' -> sid '{sid}' in namespace {self.namespace}")

    def _unregister_name(self, sid: SID) -> None:
        """
        注销sid对应的name映射
        Unregister name mapping for the given sid

        Args:
            sid (SID): 客户端连接ID / Client connection ID
        """
        # 通过session直接获取name，避免遍历映射表
        # Get name directly from session to avoid iterating through the map
        session = self.get_session(sid)
        name = session.get("name")

        if name and name in self._name_to_sid_map:
            del self._name_to_sid_map[name]
            logger.debug(f"Unregistered name '{name}' for sid '{sid}' in namespace {self.namespace}")

    def get_sid_by_name(self, name: str) -> SID | None:
        """
        通过name获取对应的sid
        Get sid by name

        Args:
            name (str): 客户端名称 / Client name

        Returns:
            SID | None: 对应的sid，如果不存在则返回None / Corresponding sid, or None if not found
        """
        return self._name_to_sid_map.get(name)

    @staticmethod
    def _extract_headers(environ: dict) -> list:
        """
        从请求环境中提取原始请求头列表
        Extract raw request headers list from request environment
        """
        headers: list = environ.get("asgi", {}).get("scope", {}).get("headers", [])
        if not headers:
            headers = environ.get("HTTP_HEADERS", [])
        return headers



# -*- coding: utf-8 -*-
"""
测试 a2c_smcp/server/sync_namespace.py
覆盖 enter_room/leave_room 及所有 on_* 分支
"""

from __future__ import annotations

from unittest.mock import MagicMock

import pytest

from a2c_smcp.server.sync_namespace import SyncSMCPNamespace
from a2c_smcp.smcp import (
    CANCEL_TOOL_CALL_NOTIFICATION,
    ENTER_OFFICE_NOTIFICATION,
    LEAVE_OFFICE_NOTIFICATION,
)


class _DummyAuthProv:
    def authenticate(self, sio, environ, auth, headers):  # pragma: no cover - not used here
        return True


@pytest.fixture()
def ns():
    n = SyncSMCPNamespace(_DummyAuthProv())
    # 伪造 server 结构
    server = MagicMock()
    server.manager = MagicMock()
    server.manager.get_participants = MagicMock(return_value=[])
    n.server = server
    # 常用桩
    n.get_session = MagicMock(return_value={})
    n.save_session = MagicMock()
    n.emit = MagicMock()
    return n


def test_enter_room_agent_rules(ns):
    # agent 在其他房间 -> 抛错
    ns.get_session.return_value = {"role": "agent", "office_id": "A"}
    with pytest.raises(ValueError):
        ns.enter_room("sidA", "B")

    # agent 不在任何房间，房间已有 agent -> 抛错
    ns.get_session.return_value = {"role": "agent"}
    ns.server.manager.get_participants.return_value = ["sidX"]
    # 该参与者为 agent

    def _get_sess_for_participant(sid):
        if sid == "sidX":
            return {"role": "agent"}
        return {}

    ns.get_session.side_effect = [
        {"role": "agent"},  # self session
        {"role": "agent"},  # participant session
    ]
    with pytest.raises(ValueError):
        ns.enter_room("sidB", "room1")

    # agent 已在同一房间 -> 返回且不重复 emit
    ns.get_session.side_effect = None
    ns.get_session.return_value = {"role": "agent", "office_id": "room1"}
    ns.emit.reset_mock()
    ns.enter_room("sidC", "room1")
    ns.emit.assert_not_called()


def test_enter_room_computer_switch_and_duplicate(ns):
    # computer 从 roomA 切到 roomB
    ns.get_session.return_value = {"role": "computer", "office_id": "roomA"}
    ns.leave_room = MagicMock()
    ns.enter_room("csid", "roomB")
    ns.leave_room.assert_called_once_with("csid", "roomA")

    # 重复加入
    ns.get_session.return_value = {"role": "computer", "office_id": "roomB"}
    ns.emit.reset_mock()
    ns.enter_room("csid", "roomB")
    ns.emit.assert_not_called()


def test_enter_room_updates_session_and_broadcast(ns):
    # 新加入应设置 office_id 并广播 ENTER_OFFICE_NOTIFICATION
    ns = SyncSMCPNamespace(_DummyAuthProv())
    server = MagicMock()
    server.manager = MagicMock()
    server.manager.get_participants = MagicMock(return_value=[])
    ns.server = server

    sess = {"role": "computer"}
    ns.get_session = MagicMock(return_value=sess)
    ns.save_session = MagicMock()
    ns.emit = MagicMock()

    ns.enter_room("sid1", "roomZ")

    assert sess["office_id"] == "roomZ"
    ns.emit.assert_called_once()
    args, kwargs = ns.emit.call_args
    assert args[0] == ENTER_OFFICE_NOTIFICATION
    assert kwargs.get("room") == "roomZ"
    assert kwargs.get("skip_sid") == "sid1"


def test_leave_room_broadcast_and_clear_session(monkeypatch):
    ns = SyncSMCPNamespace(_DummyAuthProv())
    ns.emit = MagicMock()
    sess = {"role": "computer", "office_id": "roomX"}
    ns.get_session = MagicMock(return_value=sess)
    ns.save_session = MagicMock()

    # 避免真正调用父类 leave_room
    from a2c_smcp.server.sync_base import SyncBaseNamespace

    monkeypatch.setattr(SyncBaseNamespace, "leave_room", MagicMock())
    ns.leave_room("sidX", "roomX")

    ns.emit.assert_called_once()
    args, kwargs = ns.emit.call_args
    assert args[0] == LEAVE_OFFICE_NOTIFICATION
    assert "office_id" not in sess  # 已清理


def test_on_server_join_office_ok_and_rollback_on_error():
    ns = SyncSMCPNamespace(_DummyAuthProv())
    ns.get_session = MagicMock(return_value={})
    ns.save_session = MagicMock()
    # ensure server exists for enter_room path
    server = MagicMock()
    server.manager = MagicMock()
    server.manager.get_participants = MagicMock(return_value=[])
    ns.server = server

    # 正常路径
    ok, err = ns.on_server_join_office("sid", {"role": "computer", "name": "n", "office_id": "o"})
    assert ok is True and err is None

    # enter_room 抛错 -> 回滚
    ns.enter_room = MagicMock(side_effect=RuntimeError("boom"))
    ok2, err2 = ns.on_server_join_office("sid", {"role": "computer", "name": "n", "office_id": "o"})
    assert ok2 is False and "Internal server error" in err2


def test_on_server_leave_office_ok_and_error():
    ns = SyncSMCPNamespace(_DummyAuthProv())
    ns.leave_room = MagicMock()
    ok, err = ns.on_server_leave_office("sid", {"office_id": "o"})
    assert ok is True and err is None

    ns.leave_room = MagicMock(side_effect=RuntimeError("x"))
    ok2, err2 = ns.on_server_leave_office("sid", {"office_id": "o"})
    assert ok2 is False and "Internal server error" in err2


def test_on_server_tool_call_cancel_and_update_config_and_client_paths():
    ns = SyncSMCPNamespace(_DummyAuthProv())

    # cancel 仅允许 agent
    ns.get_session = MagicMock(return_value={"role": "agent", "name": "a1"})
    ns.emit = MagicMock()
    ns.on_server_tool_call_cancel("a1", {"agent": "a1", "req_id": "r1"})
    ns.emit.assert_called_once()
    args1, kwargs1 = ns.emit.call_args
    assert args1[0] == CANCEL_TOOL_CALL_NOTIFICATION
    assert kwargs1.get("skip_sid") == "a1"

    # update_config 仅允许 computer
    ns.get_session = MagicMock(return_value={"role": "computer", "office_id": "roomR"})
    ns.emit = MagicMock()
    ns.on_server_update_config("c1", {"computer": "c1"})
    ns.emit.assert_called_once()
    _args2, kwargs2 = ns.emit.call_args
    assert kwargs2.get("room") == "roomR"

    # client tool_call：仅允许 agent，使用 call 等待响应
    ns.get_session = MagicMock(return_value={"role": "agent"})
    ns.call = MagicMock(return_value={"ok": True, "result": "success"})
    ns.get_sid_by_name = MagicMock(return_value="c1")
    ret = ns.on_client_tool_call("a1", {"robot_id": "a1", "computer": "c1", "tool_name": "t", "params": {}, "timeout": 5})
    assert ret == {"ok": True, "result": "success"}
    ns.call.assert_called_once()
    args, kwargs = ns.call.call_args
    assert kwargs.get("to") == "c1"

    # client get_tools：校验在同一房间并转发
    ns.get_session = MagicMock(
        side_effect=[
            {"role": "computer", "office_id": "room1"},  # computer sess
            {"role": "agent", "office_id": "room1"},  # agent sess
        ],
    )
    ns.call = MagicMock(
        return_value={
            "req_id": "r3",
            "tools": [
                {
                    "name": "t1",
                    "description": "d",
                    "params_schema": {"type": "object", "properties": {}, "required": []},
                    "return_schema": None,
                },
            ],
        },
    )
    ret2 = ns.on_client_get_tools("a1", {"computer": "c1", "req_id": "r3", "agent": "a1"})
    assert isinstance(ret2, dict) and ret2["req_id"] == "r3" and isinstance(ret2.get("tools"), list)
    ns.call.assert_called_once()


def test_on_server_list_room_success(monkeypatch):
    """
    测试成功列出房间内所有会话信息（同步版本）
    Test successfully listing all sessions in a room (sync version)
    """
    ns = SyncSMCPNamespace(_DummyAuthProv())
    server = MagicMock()
    ns.server = server

    # 准备测试数据：Agent 和两个 Computer 在同一房间
    # Prepare test data: Agent and two Computers in the same room
    agent_sid = "agent_1"
    comp_sid_1 = "comp_1"
    comp_sid_2 = "comp_2"
    office_id = "test_office"

    # Mock 会话数据 / Mock session data
    sessions_data = [
        {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id},
        {"sid": comp_sid_1, "name": "Computer 1", "role": "computer", "office_id": office_id},
        {"sid": comp_sid_2, "name": "Computer 2", "role": "computer", "office_id": office_id},
    ]

    # Mock get_session 返回 Agent 会话 / Mock get_session to return Agent session
    ns.get_session = MagicMock(return_value={"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id})

    # 使用 monkeypatch Mock get_all_sessions_in_office
    def mock_get_all_sessions(office_id_param, sio):
        return sessions_data

    monkeypatch.setattr("a2c_smcp.server.sync_namespace.get_all_sessions_in_office", mock_get_all_sessions)

    # 执行测试 / Execute test
    result = ns.on_server_list_room(
        agent_sid,
        {"agent": agent_sid, "req_id": "req_123", "office_id": office_id},
    )

    # 验证结果 / Verify result
    assert result["req_id"] == "req_123"
    assert len(result["sessions"]) == 3
    assert all(s["office_id"] == office_id for s in result["sessions"])
    assert any(s["sid"] == agent_sid for s in result["sessions"])
    assert any(s["sid"] == comp_sid_1 for s in result["sessions"])
    assert any(s["sid"] == comp_sid_2 for s in result["sessions"])


def test_on_server_list_room_permission_denied():
    """
    测试权限检查：Agent 只能查询自己所在房间（同步版本）
    Test permission check: Agent can only query their own room (sync version)
    """
    from a2c_smcp.smcp import ListRoomReq

    ns = SyncSMCPNamespace(_DummyAuthProv())
    server = MagicMock()
    ns.server = server

    # Agent 在 office_A，但请求查询 office_B
    # Agent is in office_A but requests to query office_B
    agent_sid = "agent_1"
    agent_session = {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": "office_A"}

    ns.get_session = MagicMock(return_value=agent_session)

    # 执行测试，应该抛出 AssertionError / Execute test, should raise AssertionError
    import pytest

    with pytest.raises(AssertionError, match="Agent只能查询自己所在房间的会话信息"):
        ns.on_server_list_room(
            agent_sid,
            {"agent": agent_sid, "req_id": "req_456", "office_id": "office_B"},
        )


def test_on_server_list_room_filters_invalid_sessions(monkeypatch):
    """
    测试过滤无效会话：只返回有效的 computer 和 agent 角色（同步版本）
    Test filtering invalid sessions: only return valid computer and agent roles (sync version)
    """
    ns = SyncSMCPNamespace(_DummyAuthProv())
    server = MagicMock()
    ns.server = server

    agent_sid = "agent_1"
    office_id = "test_office"

    # Mock 会话数据，包含一些无效的会话 / Mock session data with some invalid sessions
    sessions_data = [
        {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id},
        {"sid": "comp_1", "name": "Computer 1", "role": "computer", "office_id": office_id},
        {"sid": "invalid_1", "name": "Invalid", "role": "unknown", "office_id": office_id},  # 无效角色
        {"sid": "comp_2", "name": "Computer 2", "role": "computer", "office_id": office_id},
    ]

    agent_session = {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id}
    ns.get_session = MagicMock(return_value=agent_session)

    # 使用 monkeypatch Mock get_all_sessions_in_office
    def mock_get_all_sessions(office_id_param, sio):
        return sessions_data

    monkeypatch.setattr("a2c_smcp.server.sync_namespace.get_all_sessions_in_office", mock_get_all_sessions)

    result = ns.on_server_list_room(
        agent_sid,
        {"agent": agent_sid, "req_id": "req_789", "office_id": office_id},
    )

    # 验证结果：应该只包含 3 个有效会话（排除 unknown 角色）
    # Verify result: should only contain 3 valid sessions (excluding unknown role)
    assert len(result["sessions"]) == 3
    assert all(s["role"] in ["computer", "agent"] for s in result["sessions"])



# -*- coding: utf-8 -*-

from unittest.mock import AsyncMock, MagicMock

import pytest
from mcp import StdioServerParameters
from mcp.client.session_group import SseServerParameters, StreamableHttpParameters

from a2c_smcp.computer.mcp_clients.model import SseServerConfig, StdioServerConfig, StreamableHttpServerConfig, ToolMeta
from a2c_smcp.computer.socketio.client import SMCPComputerClient
from a2c_smcp.smcp import SMCP_NAMESPACE, UPDATE_CONFIG_EVENT


@pytest.mark.asyncio
async def test_emit_disallows_notify_and_client_events():
    """
    中文：验证 emit 校验逻辑，禁止 notify:* 与 client:* 事件
    English: Verify emit validation blocks notify:* and client:* events
    """
    client = SMCPComputerClient(computer=MagicMock())

    with pytest.raises(ValueError):
        await client.emit("notify:something", {})

    with pytest.raises(ValueError):
        await client.emit("client:something", {})


@pytest.mark.asyncio
async def test_emit_update_config_only_when_in_office(monkeypatch):
    """
    中文：仅当已加入房间（有 office_id）时触发 UPDATE_MCP_CONFIG_EVENT；未加入时不触发
    English: Fire UPDATE_MCP_CONFIG_EVENT only when office_id set; otherwise do nothing
    """
    client = SMCPComputerClient(computer=MagicMock())
    client.computer.name = "sid-123"

    sent = []

    async def fake_emit(self, event, data=None, namespace=None, callback=None):
        sent.append((event, data, namespace))

    # 注入必要上下文（无需真实连接）/ Inject minimal context (no real connection)
    client.namespaces[SMCP_NAMESPACE] = "sid-123"

    # 场景1：未加入房间，不应发送
    monkeypatch.setattr(SMCPComputerClient, "emit", fake_emit, raising=False)
    client.office_id = None
    await SMCPComputerClient.emit_update_config(client)
    assert not sent

    # 场景2：已加入房间，应发送 UPDATE_MCP_CONFIG_EVENT
    client.office_id = "office-1"
    await SMCPComputerClient.emit_update_config(client)
    assert len(sent) == 1
    assert sent[0][0] == UPDATE_CONFIG_EVENT
    assert sent[0][1] == {"computer": "sid-123"}


@pytest.mark.asyncio
async def test_on_tool_call_error_handling():
    """
    中文：当 aexecute_tool 抛出异常时，应返回 CallToolResult 且 isError=True
    English: If aexecute_tool raises, return CallToolResult with isError=True
    """
    computer = MagicMock()
    computer.aexecute_tool = AsyncMock(side_effect=RuntimeError("boom"))

    client = SMCPComputerClient(computer=computer)
    client.office_id = "office-1"
    client.computer.name = "comp-abc"

    req = {
        "computer": "comp-abc",
        "agent": "office-1",
        "req_id": "r1",
        "tool_name": "t1",
        "params": {"k": "v"},
        "timeout": 1,
    }

    ret = await client.on_tool_call(req)
    assert ret["isError"] is True
    assert ret["structuredContent"] is not None
    assert ret["structuredContent"].get("error_type") == "RuntimeError"


@pytest.mark.asyncio
async def test_on_get_config_serialization_three_types():
    """
    中文：验证 on_get_config 对 stdio/sse/streamable_http 三种配置的序列化与强校验
    English: Verify on_get_config serialization and strict validation for stdio/sse/streamable_http types
    """
    # 构造三种类型配置 / Build three types of server configs
    stdio_cfg = StdioServerConfig(
        name="stdio-srv",
        server_parameters=StdioServerParameters(command="bash", args=["-lc", "echo hi"], env={}),
        forbidden_tools=["ban1"],
        tool_meta={"toolA": ToolMeta(auto_apply=True)},
    )
    sse_cfg = SseServerConfig(
        name="sse-srv",
        server_parameters=SseServerParameters(url="http://localhost:18080/sse"),
        forbidden_tools=[],
        tool_meta={},
    )
    http_cfg = StreamableHttpServerConfig(
        name="http-srv",
        server_parameters=StreamableHttpParameters(url="http://localhost:18081"),
        forbidden_tools=[],
        tool_meta={},
    )

    class _FakeComputer:
        def __init__(self, name: str):
            self.name = name
            self._mcp_servers = (stdio_cfg, sse_cfg, http_cfg)
            self._inputs = []

        @property
        def inputs(self) -> list:
            return self._inputs

        @property
        def mcp_servers(self):
            return self._mcp_servers

    client = SMCPComputerClient(computer=_FakeComputer(name="sid-xyz"))
    client.office_id = "office-1"

    req = {"computer": "sid-xyz", "agent": "office-1", "req_id": "mock_req"}
    ret = await client.on_get_config(req)

    # 结构校验 / Structure checks
    assert "servers" in ret
    servers = ret["servers"]
    assert set(servers.keys()) == {"stdio-srv", "sse-srv", "http-srv"}

    assert servers["stdio-srv"]["type"] == "stdio"
    assert servers["sse-srv"]["type"] == "sse"
    assert servers["http-srv"]["type"] == "streamable"

    # 基础字段校验 / Base fields
    assert servers["stdio-srv"]["disabled"] is False
    assert servers["stdio-srv"]["forbidden_tools"] == ["ban1"]
    assert "toolA" in servers["stdio-srv"]["tool_meta"]

    # server_parameters 应为可序列化结构 / server_parameters should be JSON-like
    assert isinstance(servers["stdio-srv"]["server_parameters"], dict)
    assert isinstance(servers["sse-srv"]["server_parameters"], dict)
    assert isinstance(servers["http-srv"]["server_parameters"], dict)



# -*- coding: utf-8 -*-
# filename: test_model.py
# @Time    : 2025/8/18 13:35
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm

import pytest
from mcp import StdioServerParameters
from mcp.client.session_group import SseServerParameters, StreamableHttpParameters
from polyfactory.factories.pydantic_factory import ModelFactory

from a2c_smcp.computer.mcp_clients.model import (
    BaseMCPServerConfig,
    MCPServerConfig,
    SseServerConfig,
    StdioServerConfig,
    StreamableHttpServerConfig,
    ToolMeta,
)


# ----------- 工厂类生成测试数据 -----------
class ToolMetaFactory(ModelFactory):
    __model__ = ToolMeta

    @classmethod
    def auto_apply(cls) -> bool | None:
        return cls.__random__.choice([True, False, None])

    @classmethod
    def alias(cls) -> str | None:
        return cls.__random__.choice([None, f"alias_{cls.__random__.randint(1, 100)}"])


class BaseMCPServerConfigFactory(ModelFactory):
    __model__ = BaseMCPServerConfig
    __random_seed__ = 42  # 固定随机种子保证可重复性

    @classmethod
    def name(cls) -> str:
        return f"server_{cls.__random__.randint(1, 100)}"

    @classmethod
    def disabled(cls) -> bool:
        return cls.__random__.choice([True, False])

    @classmethod
    def forbidden_tools(cls) -> list:
        return [f"tool_{i}" for i in range(cls.__random__.randint(0, 5))]

    @classmethod
    def tool_meta(cls) -> dict:
        meta = {}
        tools = ToolMetaFactory.batch(size=cls.__random__.randint(1, 3))
        for index, tool_meta in enumerate(tools):
            meta[f"tool_{index}"] = tool_meta
        return meta

    @classmethod
    def vrl(cls) -> str | None:
        """生成有效的VRL脚本或None / Generate valid VRL script or None"""
        # 随机返回None或一个简单的有效VRL脚本
        # Randomly return None or a simple valid VRL script
        return cls.__random__.choice([
            None,
            '.result = "success"',
            ".transformed = true",
            '.status = "ok"',
        ])


# 使用模拟参数创建子类工厂
class StdioServerConfigFactory(BaseMCPServerConfigFactory):
    __model__ = StdioServerConfig

    @classmethod
    def server_parameters(cls) -> StdioServerParameters:
        return ModelFactory.create_factory(StdioServerParameters).build()


class SseServerConfigFactory(BaseMCPServerConfigFactory):
    __model__ = SseServerConfig

    @classmethod
    def server_parameters(cls) -> SseServerParameters:
        return ModelFactory.create_factory(SseServerParameters).build()


class StreamableHttpServerConfigFactory(BaseMCPServerConfigFactory):
    __model__ = StreamableHttpServerConfig

    @classmethod
    def server_parameters(cls) -> StreamableHttpParameters:
        return ModelFactory.create_factory(StreamableHttpParameters).build()


# ----------- 测试用例 -----------
@pytest.mark.parametrize(
    "factory",
    [ToolMetaFactory, BaseMCPServerConfigFactory, StdioServerConfigFactory, SseServerConfigFactory, StreamableHttpServerConfigFactory],
)
def test_model_creation(factory):
    """测试模型实例是否能正确创建"""
    instance = factory.build()
    assert isinstance(instance, factory.__model__)


def test_tool_meta_extra_fields():
    """测试ToolMeta的extra字段功能"""
    data = {"auto_apply": True, "extra_field": "value", "another_field": 42}
    tool_meta = ToolMeta(**data)

    # 验证额外字段被保留
    assert tool_meta.extra_field == "value"
    assert tool_meta.another_field == 42


def test_server_config_inheritance():
    """测试服务器配置类的继承关系"""
    stdio_config = StdioServerConfigFactory.build()
    assert isinstance(stdio_config, BaseMCPServerConfig)
    assert isinstance(stdio_config.server_parameters, StdioServerParameters)

    sse_config = SseServerConfigFactory.build()
    assert isinstance(sse_config, BaseMCPServerConfig)
    assert isinstance(sse_config.server_parameters, SseServerParameters)

    http_config = StreamableHttpServerConfigFactory.build()
    assert isinstance(http_config, BaseMCPServerConfig)
    assert isinstance(http_config.server_parameters, StreamableHttpParameters)


def test_mcp_config_union():
    """测试MCPServerConfig类型别名接受所有子类型"""
    servers: list[MCPServerConfig] = [
        StdioServerConfigFactory.build(),
        SseServerConfigFactory.build(),
        StreamableHttpServerConfigFactory.build(),
    ]

    assert len(servers) == 3
    assert all(isinstance(s, (StdioServerConfig, SseServerConfig, StreamableHttpServerConfig)) for s in servers)


@pytest.mark.parametrize(
    "field, value",
    [
        ("name", 123),  # 错误类型
        ("disabled", "Test"),  # 类型不匹配
        ("forbidden_tools", [1, 2, 3]),  # 列表项类型错误
        ("tool_meta", {"key": "invalid"}),  # 值类型错误
    ],
)
def test_validation_errors(field, value):
    """测试基础模型的验证错误"""
    data = {"name": "valid_name", "disabled": False, "forbidden_tools": [], "tool_meta": {}, field: value}

    with pytest.raises(ValueError):
        BaseMCPServerConfig(**data)


def test_tool_meta_ret_mapper():
    """测试ToolMeta中ret_object_mapper的特殊处理"""
    mapper = {"field": "mapped_field", "nested": {"key": "value"}}
    tool_meta = ToolMeta(alias=None, auto_apply=None, ret_object_mapper=mapper)
    assert tool_meta.ret_object_mapper == mapper


def test_server_config_with_default_tool_meta_only():
    """测试仅设置 default_tool_meta 时模型可创建并持有该字段"""
    cfg = StdioServerConfig(
        name="server_default_only",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        default_tool_meta=ToolMeta(auto_apply=True, alias=None),
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )
    assert isinstance(cfg.default_tool_meta, ToolMeta)
    assert cfg.default_tool_meta.auto_apply is True


def test_server_config_tool_meta_overrides_default():
    """测试 tool_meta 的根级字段浅覆盖 default_tool_meta"""
    default = ToolMeta(auto_apply=False, alias=None, ret_object_mapper={"a": 1})
    per_tool = {"t": ToolMeta(auto_apply=True)}
    cfg = SseServerConfig(
        name="server_with_default",
        disabled=False,
        forbidden_tools=[],
        tool_meta=per_tool,
        default_tool_meta=default,
        server_parameters=ModelFactory.create_factory(SseServerParameters).build(),
    )
    # 验证模型层面字段存在
    assert cfg.tool_meta["t"].auto_apply is True
    assert cfg.default_tool_meta.ret_object_mapper == {"a": 1}


# ----------- VRL 验证器测试用例 -----------
def test_vrl_validator_with_valid_script():
    """
    测试VRL验证器：有效的VRL脚本应该通过验证
    Test VRL validator: valid VRL script should pass validation
    """
    valid_vrl = '.result = "success"'
    cfg = StdioServerConfig(
        name="test_vrl_valid",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=valid_vrl,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )
    assert cfg.vrl == valid_vrl


def test_vrl_validator_with_invalid_script():
    """
    测试VRL验证器：无效的VRL脚本应该抛出ValueError
    Test VRL validator: invalid VRL script should raise ValueError
    """
    invalid_vrl = "this is not valid vrl syntax!!!"

    with pytest.raises(ValueError, match="VRL语法错误"):
        StdioServerConfig(
            name="test_vrl_invalid",
            disabled=False,
            forbidden_tools=[],
            tool_meta={},
            vrl=invalid_vrl,
            server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
        )


def test_vrl_validator_with_none():
    """
    测试VRL验证器：None值应该被接受
    Test VRL validator: None value should be accepted
    """
    cfg = StdioServerConfig(
        name="test_vrl_none",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=None,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )
    assert cfg.vrl is None


def test_vrl_validator_with_empty_string():
    """
    测试VRL验证器：空字符串应该被接受
    Test VRL validator: empty string should be accepted
    """
    cfg = SseServerConfig(
        name="test_vrl_empty",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl="",
        server_parameters=ModelFactory.create_factory(SseServerParameters).build(),
    )
    assert cfg.vrl == ""


def test_vrl_validator_with_whitespace_only():
    """
    测试VRL验证器：仅包含空白字符的字符串应该被接受（视为空）
    Test VRL validator: whitespace-only string should be accepted (treated as empty)
    """
    cfg = StreamableHttpServerConfig(
        name="test_vrl_whitespace",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl="   \n\t  ",
        server_parameters=ModelFactory.create_factory(StreamableHttpParameters).build(),
    )
    assert cfg.vrl == "   \n\t  "


def test_vrl_validator_with_complex_valid_script():
    """
    测试VRL验证器：复杂的有效VRL脚本应该通过验证
    Test VRL validator: complex valid VRL script should pass validation
    """
    complex_vrl = """
    .status = "processed"
    .timestamp = now()
    .transformed = true
    """
    cfg = StdioServerConfig(
        name="test_vrl_complex",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=complex_vrl,
        server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
    )
    assert cfg.vrl == complex_vrl


def test_vrl_validator_with_conditional_logic():
    """
    测试VRL验证器：带条件逻辑的VRL脚本应该通过验证
    Test VRL validator: VRL script with conditional logic should pass validation
    """
    conditional_vrl = """
    if .isError == true {
        .status = "error"
    } else {
        .status = "success"
    }
    """
    cfg = SseServerConfig(
        name="test_vrl_conditional",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=conditional_vrl,
        server_parameters=ModelFactory.create_factory(SseServerParameters).build(),
    )
    assert cfg.vrl == conditional_vrl


def test_vrl_validator_error_message_format():
    """
    测试VRL验证器：验证错误消息格式包含详细的诊断信息
    Test VRL validator: validation error message contains detailed diagnostic info
    """
    invalid_vrl = "undefined_variable_xyz"

    with pytest.raises(ValueError) as exc_info:
        StdioServerConfig(
            name="test_vrl_error_msg",
            disabled=False,
            forbidden_tools=[],
            tool_meta={},
            vrl=invalid_vrl,
            server_parameters=ModelFactory.create_factory(StdioServerParameters).build(),
        )

    error_message = str(exc_info.value)
    # 验证错误消息包含中英文提示
    assert "VRL语法错误" in error_message or "VRL syntax error" in error_message
    # 验证错误消息包含详细诊断信息（VRL Runtime提供的格式化消息）
    assert "undefined variable" in error_message.lower() or "error" in error_message.lower()


@pytest.mark.parametrize(
    "config_class,param_factory",
    [
        (StdioServerConfig, lambda: ModelFactory.create_factory(StdioServerParameters).build()),
        (SseServerConfig, lambda: ModelFactory.create_factory(SseServerParameters).build()),
        (StreamableHttpServerConfig, lambda: ModelFactory.create_factory(StreamableHttpParameters).build()),
    ],
)
def test_vrl_validator_across_all_config_types(config_class, param_factory):
    """
    测试VRL验证器：在所有配置类型中都能正常工作
    Test VRL validator: works correctly across all config types
    """
    valid_vrl = ".validated = true"

    # 测试有效脚本
    cfg_valid = config_class(
        name="test_vrl_all_types_valid",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=valid_vrl,
        server_parameters=param_factory(),
    )
    assert cfg_valid.vrl == valid_vrl

    # 测试无效脚本
    invalid_vrl = "invalid syntax here!!!"
    with pytest.raises(ValueError, match="VRL语法错误"):
        config_class(
            name="test_vrl_all_types_invalid",
            disabled=False,
            forbidden_tools=[],
            tool_meta={},
            vrl=invalid_vrl,
            server_parameters=param_factory(),
        )



# -*- coding: utf-8 -*-
# filename: test_utils.py
# @Time    : 2025/8/20 14:47
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from typing import Any

import pytest
from mcp import StdioServerParameters
from mcp.client.session_group import SseServerParameters

from a2c_smcp.computer.mcp_clients.model import SseServerConfig, StdioServerConfig
from a2c_smcp.computer.mcp_clients.sse_client import SseMCPClient
from a2c_smcp.computer.mcp_clients.stdio_client import StdioMCPClient
from a2c_smcp.computer.mcp_clients.utils import client_factory


@pytest.mark.anyio
async def test_factory(stdio_params: StdioServerParameters) -> None:
    """测试工厂函数可以正常创建"""
    std_config = StdioServerConfig(name="stdio_server", server_parameters=stdio_params)
    std_client = client_factory(std_config)
    assert isinstance(std_client, StdioMCPClient)
    assert std_client.state == "initialized"

    await std_client.aconnect()
    assert std_client.state == "connected"

    await std_client.adisconnect()
    assert std_client.state == "disconnected"


@pytest.mark.anyio
async def test_factory_multi_client(stdio_params: StdioServerParameters, sse_params: SseServerParameters, sse_server: Any) -> None:
    """测试工厂函数可以正常依次创建与关闭多个客户端"""
    std_config = StdioServerConfig(name="stdio_server", server_parameters=stdio_params)
    std_client = client_factory(std_config)
    assert isinstance(std_client, StdioMCPClient)
    assert std_client.state == "initialized"

    await std_client.aconnect()
    assert std_client.state == "connected"

    await std_client.adisconnect()
    assert std_client.state == "disconnected"

    sse_config = SseServerConfig(name="sse_server", server_parameters=sse_params)
    sse_client = client_factory(sse_config)
    assert isinstance(sse_client, SseMCPClient)
    assert sse_client.state == "initialized"

    await sse_client.aconnect()
    assert sse_client.state == "connected"

    await sse_client.adisconnect()
    assert sse_client.state == "disconnected"


# 测试两个不同客户端依次创建，然后再依次连接，再依次断开
# 参考： tests/unit_tests/utils/test_async_exit_stack_anyio.py 因为MCP内部使用了anyio的task_group，因此退出时必须按入栈顺序退出，否则会出错
@pytest.mark.anyio
async def test_factory_multi_client2(stdio_params: StdioServerParameters, sse_params: SseServerParameters, sse_server: Any) -> None:
    std_config = StdioServerConfig(name="stdio_server", server_parameters=stdio_params)
    std_client = client_factory(std_config)
    assert isinstance(std_client, StdioMCPClient)
    assert std_client.state == "initialized"

    sse_config = SseServerConfig(name="sse_server", server_parameters=sse_params)
    sse_client = client_factory(sse_config)
    assert isinstance(sse_client, SseMCPClient)
    assert sse_client.state == "initialized"

    await std_client.aconnect()
    assert std_client.state == "connected"

    await sse_client.aconnect()
    assert sse_client.state == "connected"

    await std_client.adisconnect()
    assert std_client.state == "disconnected"

    await sse_client.adisconnect()
    assert sse_client.state == "disconnected"



from __future__ import annotations

import pytest

from a2c_smcp.computer.cli.utils import parse_kv_pairs, resolve_import_target


class TestParseKvPairs:
    def test_none_and_empty(self) -> None:
        assert parse_kv_pairs(None) is None
        assert parse_kv_pairs("") is None
        assert parse_kv_pairs("   ") is None

    def test_basic_key_value_parsing(self) -> None:
        assert parse_kv_pairs("a:1,b:2") == {"a": "1", "b": "2"}
        assert parse_kv_pairs(" a : 1 , b : 2 ") == {"a": "1", "b": "2"}
        # ignore empty segments
        assert parse_kv_pairs("a:1, ,b:2,,") == {"a": "1", "b": "2"}

    def test_error_missing_colon_and_empty_key(self) -> None:
        with pytest.raises(ValueError):
            parse_kv_pairs("a1,b2")
        with pytest.raises(ValueError):
            parse_kv_pairs(":v")

    def test_json_object_string(self) -> None:
        # should support valid JSON object string and keep value types
        d = parse_kv_pairs('{"x": 1, "y": "2", "z": true}')
        assert d == {"x": 1, "y": "2", "z": True}

    def test_json_non_object_should_error(self) -> None:
        with pytest.raises(ValueError):
            parse_kv_pairs("[1,2,3]")
        with pytest.raises(ValueError):
            parse_kv_pairs('"str"')


class TestResolveImportTarget:
    def test_resolve_with_colon(self) -> None:
        obj = resolve_import_target("a2c_smcp.computer.cli.utils:resolve_import_target")
        assert obj is resolve_import_target

    def test_resolve_with_dot(self) -> None:
        obj = resolve_import_target("a2c_smcp.computer.cli.utils.resolve_import_target")
        assert obj is resolve_import_target

    def test_relative_import_not_allowed(self) -> None:
        with pytest.raises(ValueError):
            resolve_import_target(".cli.utils:resolve_import_target")

    def test_invalid_target_format(self) -> None:
        with pytest.raises(ValueError):
            resolve_import_target("only_module_name")

    def test_missing_attribute(self) -> None:
        with pytest.raises(AttributeError):
            resolve_import_target("a2c_smcp.computer.cli.utils:not_exist_attr")



"""
* 文件名: mock_socketio_server
* 作者: JQQ
* 创建日期: 2025/8/21
* 最后修改日期: 2025/9/30
* 版权: 2023 JQQ. All rights reserved.
* 依赖: socketio
* 描述: 基于标准Server命名空间实现的测试用Mock / Test Mock based on standard Server namespace
"""

from typing import Any

from socketio import AsyncServer

from a2c_smcp.server.auth import AuthenticationProvider
from a2c_smcp.server.namespace import SMCPNamespace
from a2c_smcp.server.sync_namespace import SyncSMCPNamespace
from a2c_smcp.smcp import SMCP_NAMESPACE
from a2c_smcp.utils.logger import logger


class _PassAuthenticationProvider(AuthenticationProvider):
    """
    简易认证提供者：测试环境放行所有连接
    Simple auth provider: allow all connections in tests
    """

    async def authenticate(self, sio, environ, auth, headers):  # type: ignore[override]
        return True


class MockComputerServerNamespace(SMCPNamespace):
    """
    基于标准SMCPNamespace的测试命名空间，保持原有测试记录能力
    Test namespace based on standard SMCPNamespace, preserving action recording for tests
    """

    def __init__(self) -> None:
        super().__init__(auth_provider=_PassAuthenticationProvider())
        # 记录客户端关键操作，供断言使用
        # Record client key operations for assertions
        self.client_operations_record: dict[str, tuple[str, Any]] = {}

    async def on_connect(self, sid: str, environ: dict, auth: dict | None = None) -> bool:  # type: ignore[override]
        # 先执行标准认证与连接流程，再记录
        # Execute standard auth/connection then record
        result = await super().on_connect(sid, environ, auth)
        self.client_operations_record[sid] = ("connect", None)
        logger.info(f"Client {sid} 已连接 / connected")
        return result

    async def on_disconnect(self, sid: str) -> None:  # type: ignore[override]
        self.client_operations_record[sid] = ("disconnect", None)
        await super().on_disconnect(sid)

    async def enter_room(self, sid: str, room: str, namespace: str | None = None) -> None:  # type: ignore[override]
        await super().enter_room(sid, room, namespace)
        self.client_operations_record[sid] = ("enter_room", room)

    async def leave_room(self, sid: str, room: str, namespace: str | None = None) -> None:  # type: ignore[override]
        self.client_operations_record[sid] = ("leave_room", room)
        await super().leave_room(sid, room, namespace)

    async def on_server_join_office(self, sid: str, data):  # type: ignore[override]
        # 记录加入办公室事件 / record join office event
        self.client_operations_record[sid] = ("server_join_office", data)
        return await super().on_server_join_office(sid, data)

    async def on_server_leave_office(self, sid: str, data):  # type: ignore[override]
        # 记录离开办公室事件 / record leave office event
        self.client_operations_record[sid] = ("server_leave_office", data)
        return await super().on_server_leave_office(sid, data)

    async def on_server_update_config(self, sid: str, data):  # type: ignore[override]
        # 记录更新配置事件 / record update config event
        self.client_operations_record[sid] = ("server_update_config", data)
        return await super().on_server_update_config(sid, data)


class MockComputerServerSyncNamespace(SyncSMCPNamespace):
    """
    同步版本的测试命名空间
    Synchronous version test namespace
    """

    def __init__(self) -> None:
        super().__init__(auth_provider=_PassAuthenticationProvider())
        self.client_operations_record: dict[str, tuple[str, Any]] = {}

    def on_connect(self, sid: str, environ: dict, auth: dict | None = None) -> bool:  # type: ignore[override]
        result = super().on_connect(sid, environ, auth)
        self.client_operations_record[sid] = ("connect", None)
        logger.info(f"Client {sid} 已连接 / connected (sync)")
        return result

    def on_disconnect(self, sid: str) -> None:  # type: ignore[override]
        self.client_operations_record[sid] = ("disconnect", None)
        super().on_disconnect(sid)

    def enter_room(self, sid: str, room: str, namespace: str | None = None) -> None:  # type: ignore[override]
        super().enter_room(sid, room, namespace)
        self.client_operations_record[sid] = ("enter_room", room)

    def leave_room(self, sid: str, room: str, namespace: str | None = None) -> None:  # type: ignore[override]
        self.client_operations_record[sid] = ("leave_room", room)
        super().leave_room(sid, room, namespace)


def create_computer_test_socketio() -> AsyncServer:
    """
    创建用于测试的Socket.IO服务器（异步）
    Create Async Socket.IO server for tests
    """
    sio = AsyncServer(
        async_mode="asgi",
        logger=True,
        engineio_logger=True,
        cors_allowed_origins="*",
        ping_timeout=10,
        ping_interval=10,
        async_handlers=True,
    )

    namespace = MockComputerServerNamespace()
    sio.register_namespace(namespace)

    # 兼容旧引用路径：通过命名空间映射可在测试中按 SMCP_NAMESPACE 取回
    # Backward compatible: can fetch namespace via SMCP_NAMESPACE in tests
    assert SMCP_NAMESPACE in (namespace.namespace,)

    return sio
